package cn.com.qmth.examcloud.core.oe.student.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.Util;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.StudentBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentResp;
import cn.com.qmth.examcloud.core.oe.common.base.Constants;
import cn.com.qmth.examcloud.core.oe.common.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamFileAnswerTempEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamProperties;
import cn.com.qmth.examcloud.core.oe.common.enums.HandInExamType;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.common.service.ExamScoreNoticeQueueService;
import cn.com.qmth.examcloud.core.oe.student.api.request.GetStudentOnlineExamInfoReq;
import cn.com.qmth.examcloud.core.oe.student.api.response.GetStudentOnlineExamInfoResp;
import cn.com.qmth.examcloud.core.oe.student.bean.*;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamProcessResultInfo;
import cn.com.qmth.examcloud.core.oe.student.controller.bean.BatchGetUpyunSignDomain;
import cn.com.qmth.examcloud.core.oe.student.controller.bean.BatchGetUpyunSignDomainQuery;
import cn.com.qmth.examcloud.core.oe.student.controller.bean.GetUpyunSignDomain;
import cn.com.qmth.examcloud.core.oe.student.controller.bean.GetUpyunSignDomainQuery;
import cn.com.qmth.examcloud.core.oe.student.service.ExamControlService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamSessionInfoService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamStudentService;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.helpers.SequenceLockHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import cn.com.qmth.examcloud.web.upyun.UpYunHttpRequest;
import cn.com.qmth.examcloud.web.upyun.UpyunPathEnvironmentInfo;
import cn.com.qmth.examcloud.web.upyun.UpyunService;
import com.mysql.cj.util.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author chenken
 * @date 2018年8月13日 下午2:28:08
 * @company QMTH
 * @description 在线考试控制Controller
 */
@Api(tags = "在线考试控制")
@RestController
@RequestMapping("${app.api.oe.student}/examControl")
public class ExamControlController extends ControllerSupport {
    //抓拍照片的又拍云id
    private static final String CAPTURE_PHOTO_UPYUN_SITEID = "capturePhoto";
    @Autowired
    StudentCloudService studentCloudService;
    @Autowired
    ExamRecordDataRepo examRecordDataRepo;
    @Autowired
    private ExamStudentService examStudentService;
    @Autowired
    private ExamControlService examControlService;
    @Autowired
    private ExamScoreNoticeQueueService examScoreNoticeQueueService;
    @Autowired
    UpyunService upyunService;
    @Value("${$upyun.site.1.domain}")
    private String upyunFileUrl;
    @Autowired
    RedisClient redisClient;
    @Autowired
    ExamRecordDataService examRecordDataService;
    @Autowired
    private ExamSessionInfoService examSessionInfoService;

    /**
     * 获取当前服务器时间
     *
     * @return
     */
    @ApiOperation(value = "获取当前服务器时间")
    @GetMapping("/currentTime")
    public Long getCurrentTime() {
        return new Date().getTime();
    }

    /**
     * 获取在线考试待考列表
     *
     * @return
     */
    @ApiOperation(value = "获取在线考试待考列表")
    @GetMapping("/queryExamList")
    public List<ExamStudentInfo> queryExamList() {
        User user = getAccessUser();
        return examStudentService.queryOnlineExamList(user.getUserId());
    }

    /**
     * 开始考试
     */
    @ApiOperation(value = "开始考试")
    @GetMapping("/startExam")
    public StartExamInfo startExam(@RequestParam Long examStudentId) {
        User user = getAccessUser();
        String sequenceLockKey = Constants.START_EXAM_LOCK_PREFIX + user.getUserId();
        StartExamInfo startExamInfo;
        //开始考试上锁,分布式锁,系统在请求结束后会，自动释放锁，无需手动解锁
        SequenceLockHelper.getLock(sequenceLockKey);
        Check.isNull(examStudentId, "examStudentId不能为空");

        startExamInfo = examControlService.startExam(examStudentId, user);
        return startExamInfo;
    }

    /**
     * 结束考试：交卷..
     */
    @ApiOperation(value = "结束考试：交卷")
    @GetMapping("/endExam")
    public void endExam() {
        User user = getAccessUser();
        Long studentId = user.getUserId();
        String sequenceLockKey = Constants.END_EXAM_LOCK_PREFIX + studentId;
        //系统在请求结束后会，自动释放锁，无需手动解锁
        SequenceLockHelper.getLock(sequenceLockKey);

        long st = System.currentTimeMillis();
        long startTime = System.currentTimeMillis();

        // 获取考试会话，判断考生是否已结束考试
        ExamSessionInfo examSessionInfo = examSessionInfoService.getExamSessionInfo(studentId);
        if (examSessionInfo == null) {
            throw new StatusException("oestudent-100100", "考试会话已过期");
        }


        if (log.isDebugEnabled()) {
            log.debug("0 [END_EXAM] 交卷前处理耗时:" + (System.currentTimeMillis() - startTime) + " ms");
        }
        examControlService.handInExam(examSessionInfo.getExamRecordDataId(), HandInExamType.MANUAL);
        if (log.isDebugEnabled()) {
            log.debug("1 [END_EXAM]合计 耗时:" + (System.currentTimeMillis() - st) + " ms");
        }
    }

    /**
     * 交卷后续处理
     */
    @ApiOperation(value = "交卷后续处理")
    @GetMapping("/processAfterEndExam")
    public ExamProcessResultInfo processAfterEndExam(@RequestParam Long examRecordDataId) {
        User user = getAccessUser();
        ExamProcessResultInfo res = new ExamProcessResultInfo();
        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId,
                ExamRecordDataEntity.class);
        try {
            examControlService.processAfterHandInExam(examRecordDataId, examRecordData.getStudentId(), HandInExamType.MANUAL);
        } catch (StatusException e) {
            if (e.getCode().equals(Constants.CAPTURE_PROCESSING_STATUS_CODE)) {
                res.setCode(Constants.PROCESSING_EXAM_RECORD_CODE);
                return res;
            }
            throw e;
        } catch (Exception e) {
            throw e;
        }


        //根据相关条件添加推分队列
        Long examId = examRecordData.getExamId();
        String isPushScore = CacheHelper.getExamOrgProperty(examId, examRecordData.getOrgId(),
                ExamProperties.PUSH_SCORE.name()).getValue();
        if (isPushScore != null && Constants.isTrue.equals(isPushScore)) {
            //保存分数通知队列(只有考试设置中,配置为允许推分的才发送通知)
            boolean isExistNoticeQueue = examScoreNoticeQueueService.isExistExamScoreNoticeQueue(user.getRootOrgId());
            //如果当前组织机构的通知队列数据不存在则保存至队列，否则不作任何处理
            if (!isExistNoticeQueue) {
                examScoreNoticeQueueService.addExamScoreNoticeQueue(user.getRootOrgId());
            }
        }
        res.setCode(Constants.COMMON_SUCCESS_CODE);
        return res;
    }

    @ApiOperation(value = "获取考试记录信息")
    @GetMapping("/getEndExamInfo")
    public EndExamInfo getEndExamInfo(@RequestParam Long examRecordDataId) {
        return examControlService.getEndExamInfo(examRecordDataId);
    }

    /**
     * 考试心跳
     *
     * @return 剩余时间
     */
    @ApiOperation(value = "考试心跳")
    @GetMapping("/examHeartbeat")
    public Long examHeartbeat() {
        User user = getAccessUser();
        return examControlService.examHeartbeat(user);
    }

    /**
     * 断点续考:检查正在进行中的考试
     */
    @ApiOperation(value = "断点续考:检查正在进行中的考试")
    @GetMapping("/checkExamInProgress")
    public ExamProcessResultInfo checkExamInProgress() {
        User user = getAccessUser();
        String sequenceLockKey = Constants.START_EXAM_LOCK_PREFIX + user.getUserId();
        //系统在请求结束后会，自动释放锁，无需手动解锁
        SequenceLockHelper.getLock(sequenceLockKey);
        ExamProcessResultInfo res = new ExamProcessResultInfo();
        try {
            CheckExamInProgressInfo info = examControlService.checkExamInProgress(user.getUserId());
            res.setCode(Constants.COMMON_SUCCESS_CODE);
            res.setData(info);
            return res;
        } catch (StatusException e) {
            if (e.getCode().equals(Constants.EXAM_RECORD_NOT_END_STATUS_CODE)) {
                res.setCode(Constants.PROCESSING_EXAM_RECORD_CODE);
                return res;
            }
            throw e;
        } catch (Exception e) {
            throw e;
        }
    }

    /**
     * 获取又拍云文件上传签名（微信小程序调用）
     */
    @ApiOperation(value = "获取又拍云文件上传签名（微信小程序调用）")
    @PostMapping("/upyunSignature")
    public UpyunSignatureInfo getUpyunSignature(@ModelAttribute @Valid GetUpyunSignatureReq req) {
        return examControlService.getUpyunSignature(req);
    }

    /**
     * 校验二维码（微信小程序调用）
     */
    @Naked
    @ApiOperation(value = "校验二维码（微信小程序调用）")
    @PostMapping("/checkQrCode")
    public CheckQrCodeInfo checkQrCode(@RequestParam(required = true) String qrCode) {
        return examControlService.checkQrCode(qrCode);
    }

    /**
     * 保存上传的文件（微信小程序调用）
     */
    @ApiOperation(value = "保存上传的文件（微信小程序调用）")
    @PostMapping("/saveUploadedFile")
    public Long saveUploadedFile(@ModelAttribute @Valid SaveUploadedFileReq req) {
        User user = getAccessUser();
        ExamFileAnswerTempEntity examFileAnswerTempEntity = examControlService.saveUploadedFile(req, user);
        try {
            String fileUrl = "";
            if (req.getFilePath().indexOf(",") > -1) {
                for (String url : req.getFilePath().split(",")) {
                    fileUrl += upyunFileUrl + url + ",";
                }
                fileUrl = fileUrl.substring(0, fileUrl.length() - 1);
            } else {
                fileUrl = upyunFileUrl + req.getFilePath();
            }
            examControlService.sendFileAnswerToWebSocket(req.getExamRecordDataId(), req.getOrder(),
                    fileUrl, req.getTransferFileType(), user.getUserId());
        } catch (Exception e) {
            examControlService.deleteExamFileAnswerTemp(req);
            throw new StatusException("100009", "消息通知失败", e);
        }
        return examFileAnswerTempEntity.getId();
    }

    /**
     * 查询客户端对上传的文件的响应状态（微信小程序调用）
     */
    @ApiOperation(value = "查询客户端对上传的文件的响应状态（微信小程序调用）")
    @PostMapping("/getUploadedFileAcknowledgeStatus")
    public String getUploadedFileAcknowledgeStatus(@ModelAttribute @Valid GetUploadedFileAcknowledgeStatusReq req) {
        return examControlService.getUploadedFileAcknowledgeStatus(req);
    }

    /**
     * 修改上传音频结果推送状态
     */
    @ApiOperation(value = "修改上传音频结果推送状态")
    @PostMapping("/saveUploadedFileAcknowledgeStatus")
    public void saveUploadedFileAcknowledgeStatus(@RequestBody @Valid SaveUploadedFileAcknowledgeStatusReq req) {
        examControlService.saveUploadedFileAcknowledgeStatus(req);
    }

    @Naked
    @ApiOperation(value = "获取学生是否有正在进行的网考信息")
    @PostMapping("/getStudentOnLineExamInfo")
    public GetStudentOnlineExamInfoResp getStudentOnLineExamInfo(@RequestBody GetStudentOnlineExamInfoReq req) {
        if (null == req.getRootOrgId()) {
            throw new StatusException("101001", "组织机构id参数不允许为空");
        }
        if (StringUtils.isNullOrEmpty(req.getStudentCode()) && StringUtils.isNullOrEmpty(req.getIdentityNumber())
                && StringUtils.isNullOrEmpty(req.getPhone())) {
            throw new StatusException("101002", "学生学号、身份证号或手机号参数至少有一个不为空");
        }
        GetStudentReq getStudentReq = new GetStudentReq();
        getStudentReq.setRootOrgId(req.getRootOrgId());
        if (!StringUtils.isNullOrEmpty(req.getStudentCode())) {
            getStudentReq.setStudentCode(req.getStudentCode());
        }
        if (!StringUtils.isNullOrEmpty(req.getIdentityNumber())) {
            getStudentReq.setIdentityNumber(req.getIdentityNumber());
        }
        if (!StringUtils.isNullOrEmpty(req.getPhone())) {
            getStudentReq.setSecurityPhone(req.getPhone());
        }
        //此方法内部已作空处理，不需要再判断非空
        GetStudentResp studentResp = studentCloudService.getStudent(getStudentReq);
        StudentBean studentInfo = studentResp.getStudentInfo();
        Long studentId = studentInfo.getId();
        ExamRecordDataEntity examRecordData = examRecordDataRepo.findOnlineExamingRecordByStudentId(studentId);
        GetStudentOnlineExamInfoResp resp = new GetStudentOnlineExamInfoResp();
        if (null != examRecordData && null != examRecordData.getId()) {
            resp.setExistExamingRecord(true);
        } else {
            resp.setExistExamingRecord(false);
        }
        return resp;
    }

    @ApiOperation(value = "获取抓拍照片的又拍云签名")
    @GetMapping("/getCapturePhotoUpYunSign")
    public GetUpyunSignDomain getCapturePhotoUpYunSign(GetUpyunSignDomainQuery query) {
        String fileSuffix = query.getFileSuffix();
        if (StringUtils.isNullOrEmpty(fileSuffix)) {
            throw new StatusException("200001", "文件后缀名不允许为空");
        }
        fileSuffix = fileSuffix.indexOf(".") == -1 ? "." + fileSuffix : fileSuffix;

        GetUpyunSignDomain result = new GetUpyunSignDomain();
        User accessUser = this.getAccessUser();
        String signIdentifier = String.valueOf(System.currentTimeMillis());
        String upyunSignRedisKey = Constants.EXAM_CAPTURE_PHOTO_UPYUN_SIGN_PREFIX
                + accessUser.getUserId() + "_" + signIdentifier;

        UpyunPathEnvironmentInfo env = new UpyunPathEnvironmentInfo();
        env.setRootOrgId(accessUser.getRootOrgId().toString());
        env.setUserId(accessUser.getUserId().toString());
        env.setFileSuffix(fileSuffix);
        UpYunHttpRequest upYunHttpRequest = upyunService.buildUpYunHttpRequest(CAPTURE_PHOTO_UPYUN_SITEID, env, query.getFileMd5());
        redisClient.set(upyunSignRedisKey, upYunHttpRequest, 60);
        result.setAccessUrl(upYunHttpRequest.getAccessUrl());
        result.setFormUrl(upYunHttpRequest.getFormUrl());
        result.setFormParams(upYunHttpRequest.getFormParams());
        result.setSignIdentifier(signIdentifier);
        return result;
    }

    @ApiOperation(value = "批量获取抓拍照片的又拍云签名")
    @PostMapping("/batchGetCapturePhotoUpYunSign")
    public BatchGetUpyunSignDomain batchGetCapturePhotoUpYunSign(@RequestBody BatchGetUpyunSignDomainQuery batchQuery) {
        if (batchQuery.getQueryList() == null || batchQuery.getQueryList().isEmpty()) {
            throw new StatusException("200001", "查询条件不允许为空");
        }

        List<GetUpyunSignDomain> signDomainList = new ArrayList<>();
        for (GetUpyunSignDomainQuery query : batchQuery.getQueryList()) {
            String fileSuffix = query.getFileSuffix();
            if (StringUtils.isNullOrEmpty(fileSuffix)) {
                throw new StatusException("", "文件后缀名不允许为空");
            }
            fileSuffix = fileSuffix.indexOf(".") == -1 ? "." + fileSuffix : fileSuffix;

            GetUpyunSignDomain upyunSignDomain = new GetUpyunSignDomain();
            User accessUser = this.getAccessUser();
            String signIdentifier = String.valueOf(System.currentTimeMillis());
            String upyunSignRedisKey = Constants.EXAM_CAPTURE_PHOTO_UPYUN_SIGN_PREFIX
                    + accessUser.getUserId() + "_" + signIdentifier;

            UpyunPathEnvironmentInfo env = new UpyunPathEnvironmentInfo();
            env.setRootOrgId(accessUser.getRootOrgId().toString());
            env.setUserId(accessUser.getUserId().toString());
            env.setFileSuffix(fileSuffix);
            UpYunHttpRequest upYunHttpRequest = upyunService.buildUpYunHttpRequest(CAPTURE_PHOTO_UPYUN_SITEID, env, query.getFileMd5());
            redisClient.set(upyunSignRedisKey, upYunHttpRequest, 60);
            upyunSignDomain.setAccessUrl(upYunHttpRequest.getAccessUrl());
            upyunSignDomain.setFormUrl(upYunHttpRequest.getFormUrl());
            upyunSignDomain.setFormParams(upYunHttpRequest.getFormParams());
            upyunSignDomain.setSignIdentifier(signIdentifier);

            signDomainList.add(upyunSignDomain);
            Util.sleep(TimeUnit.MILLISECONDS,1);
        }
        BatchGetUpyunSignDomain result = new BatchGetUpyunSignDomain();
        result.setList(signDomainList);
        return result;
    }
}
