package cn.com.qmth.examcloud.core.oe.student.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.common.base.Constants;
import cn.com.qmth.examcloud.core.oe.common.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamFaceLivenessVerifyEntity;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamFaceLivenessVerifyRepo;
import cn.com.qmth.examcloud.core.oe.student.bean.GetFaceVerifyTokenInfo;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamProcessResultInfo;
import cn.com.qmth.examcloud.core.oe.student.service.ExamFaceLivenessVerifyService;
import cn.com.qmth.examcloud.core.oe.websocket.api.WebsocketCloudService;
import cn.com.qmth.examcloud.core.oe.websocket.api.request.SendMessageReq;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * @author chenken
 * @date 2018年9月5日 下午6:28:34
 * @company QMTH
 * @description 活体检测controller
 */
@Api(tags = "活体检测")
@RestController
@RequestMapping("${app.api.oe.student}/examFaceLivenessVerify")
public class ExamFaceLivenessVerifyController extends ControllerSupport {

    @Autowired
    private ExamFaceLivenessVerifyService examFaceLivenessVerifyService;

    @Autowired
    private WebsocketCloudService websocketCloudService;

    @Autowired
    private ExamFaceLivenessVerifyRepo examFaceLivenessVerifyRepo;


    @ApiOperation(value = "检测学生底照是否能获取到faceId验证的token")
    @GetMapping("/checkFaceLiveness")
    public GetFaceVerifyTokenInfo checkFaceLiveness() {
        User user = getAccessUser();
        String bizNo = user.getUserId().toString();
        return examFaceLivenessVerifyService.getFaceVerifyToken(user.getUserId(), bizNo);
    }

    /**
     * 获得一个faceid用于网页端活体检测的token
     *
     * @param examRecordDataId
     * @return
     */
    @ApiOperation(value = "获得一个faceid用于网页端活体检测的token")
    @GetMapping("/getFaceLivenessVerifyToken/{examRecordDataId}")
    public GetFaceVerifyTokenInfo getFaceVerifyToken(@PathVariable Long examRecordDataId) {
        Check.isNull(examRecordDataId, "examRecordDataId不能为空");
        User user = getAccessUser();
        ExamFaceLivenessVerifyEntity faceVerify = examFaceLivenessVerifyService.saveFaceVerifyByExamRecordDataId(examRecordDataId);
        GetFaceVerifyTokenInfo getFaceVerifyTokenInfo = examFaceLivenessVerifyService.getFaceVerifyToken(user.getUserId(), faceVerify.getId().toString());
        if (!getFaceVerifyTokenInfo.getSuccess()) {
            faceVerify.setIsError(true);
            faceVerify.setErrorMsg(getFaceVerifyTokenInfo.getErrorMsg());
            examFaceLivenessVerifyRepo.save(faceVerify);
        }
        return getFaceVerifyTokenInfo;
    }

    @ApiOperation(value = "更新活体检测结果")
    @GetMapping("/updateFaceLivenessVerify/{examRecordDataId}")
    public void updateFaceVerify(@PathVariable Long examRecordDataId, @RequestParam String errorMsg) {
        Check.isNull(examRecordDataId, "examRecordDataId不能为空");
        List<ExamFaceLivenessVerifyEntity> examFaceLivenessVerifyEntities = examFaceLivenessVerifyService.listFaceVerifyByExamRecordId(examRecordDataId);
        if (examFaceLivenessVerifyEntities != null && examFaceLivenessVerifyEntities.size() > 0) {
            ExamFaceLivenessVerifyEntity examFaceLivenessVerifyEntity = examFaceLivenessVerifyEntities.get(0);
            examFaceLivenessVerifyEntity.setIsError(true);
            examFaceLivenessVerifyEntity.setErrorMsg(errorMsg);
            examFaceLivenessVerifyRepo.save(examFaceLivenessVerifyEntity);
        }
    }

    /**
     * 人脸验证完成后的回调，由faceId调用
     *
     * @param data
     * @throws Exception
     */
    @Naked
    @ApiOperation(value = "人脸验证完成后的回调，由faceId调用")
    @PostMapping("/faceLivenessVerifyCallback")
    public void faceLivenessVerifyCallback(@RequestParam String data) throws Exception {
        log.info("faceId回调,data=" + data);

        JSONObject returnJsonObject = new JSONObject(data);
        Long faceVerifyId = Long.parseLong(returnJsonObject.get("biz_no") + "");
        ExamFaceLivenessVerifyEntity currentFaceVerify = examFaceLivenessVerifyService.findFaceVerifyById(faceVerifyId);
        /**
         * 如果该检测记录结果已经非空了,直接返回，
         * 有可能超时程序已经将结果填成TIME_OUT了
         */
        if (currentFaceVerify.getVerifyResult() != null) {
            return;
        }

        ExamFaceLivenessVerifyEntity faceVerify = examFaceLivenessVerifyService.faceIdNotify(data);
        List<ExamFaceLivenessVerifyEntity> faceVerifies = examFaceLivenessVerifyService.listFaceVerifyByExamRecordId(faceVerify.getExamRecordDataId());
        JSONObject jsonObject = new JSONObject();
        //验证次数
        jsonObject.put("verifyCount", faceVerifies.size());
        //取最后一次验证结果
        jsonObject.put("verifyResult", faceVerifies.get(faceVerifies.size() - 1).getVerifyResult().name());
        jsonObject.put("examRecordDataId", faceVerify.getExamRecordDataId());

        SendMessageReq sendMessageReq = new SendMessageReq();
        sendMessageReq.setExamRecordDataId(faceVerify.getExamRecordDataId());
        sendMessageReq.setReturnMsgJson(jsonObject.toString());
        websocketCloudService.sendMessage(sendMessageReq);
    }

    /**
     * 人脸检测超时处理
     *
     * @param examRecordDataId
     * @return
     */
    @ApiOperation(value = "人脸检测超时处理")
    @GetMapping("/faceLivenessVerifyTimeOut/{examRecordDataId}")
    public String faceTestTimeOut(@PathVariable Long examRecordDataId) {
        JSONObject jsonObject = new JSONObject();
        examFaceLivenessVerifyService.faceTestTimeOut(examRecordDataId);
        List<ExamFaceLivenessVerifyEntity> faceVerifies = examFaceLivenessVerifyService.listFaceVerifyByExamRecordId(examRecordDataId);
        //验证次数
        try {
            jsonObject.put("verifyCount", faceVerifies.size());
            //取最后一次验证结果
            jsonObject.put("verifyResult", faceVerifies.get(faceVerifies.size() - 1).getVerifyResult().name());
            jsonObject.put("examRecordDataId", examRecordDataId);
        } catch (JSONException e) {
            e.printStackTrace();
        }

        return jsonObject.toString();
    }

    /**
     * 人脸活体检测结束处理
     *
     * @param examRecordDataId 考试记录id
     * @param result           活体检测结果
     * @throws Exception
     */
    @ApiOperation(value = "人脸检测结束处理")
    @GetMapping(value = "faceLivenessVerifyEnd/{examRecordDataId}")
    public ExamProcessResultInfo faceTestEndHandle(@PathVariable Long examRecordDataId, @RequestParam String result) throws Exception {
        ExamProcessResultInfo res = new ExamProcessResultInfo();
        try {
            User user = getAccessUser();
            examFaceLivenessVerifyService.faceTestEndHandle(examRecordDataId, user.getUserId(), result);
            res.setCode(Constants.COMMON_SUCCESS_CODE);
            return res;
        } catch (StatusException e) {
            if (e.getCode().equals(Constants.EXAM_RECORD_NOT_END_STATUS_CODE)) {
                res.setCode(Constants.PROCESSING_EXAM_RECORD_CODE);
                return res;
            }
            throw e;
        } catch (Exception e) {
            throw e;
        }
    }

}
