package cn.com.qmth.examcloud.core.oe.student.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.core.oe.common.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamQuestionEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordQuestionsEntity;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamSessionInfo;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamStudentQuestionInfo;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamSessionInfoService;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestionStructure;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author  	chenken
 * @date    	2018年9月5日 下午4:57:05
 * @company 	QMTH
 * @description 考试作答记录controller
 */
@Api(tags = "考试过程中-试题相关接口")
@RestController
@RequestMapping("${app.api.oe.student}/examQuestion")
public class ExamQuestionController extends ControllerSupport {
	
	@Autowired
	private ExamRecordQuestionsService examRecordQuestionsService;
	
	@Autowired
    private ExamSessionInfoService examSessionInfoService;

	/**
	 * 将mongodb中的答过的题和redis中的题目列表合并返回给前端
	 * 返回给前端时注意将正确答案和得分置成null
	 * @return
	 */
	@ApiOperation(value = "考试过程中-获取试题列表")
	@GetMapping("/findExamQuestionList")
	public List<ExamQuestionEntity> findExamQuestionList(){
		User user = getAccessUser();
		ExamSessionInfo examSessionInfo = examSessionInfoService.getExamSessionInfo(user.getUserId());
		if (null== examSessionInfo){
			throw new StatusException("500001","考试会话已过期，请重新开考");
		}
		ExamRecordQuestionsEntity examRecordQuestionsEntity = examRecordQuestionsService.
				getExamRecordQuestionsAndFixExamRecordDataIfNecessary(examSessionInfo.getExamRecordDataId());

		List<ExamQuestionEntity> examQuestionList = examRecordQuestionsEntity.getExamQuestionEntities();
		for(ExamQuestionEntity examQuestion:examQuestionList){
			examQuestion.setCorrectAnswer(null);
			examQuestion.setStudentScore(null);
		}
		return examQuestionList;
	}

	/**
	 * 获取试题内容
	 * @param questionId
	 * @return
	 */
	@ApiOperation(value = "考试过程中-获取试题内容")
	@GetMapping("/getQuestionContent")
	public String getQuestionContent(@RequestParam String questionId){
		User user = getAccessUser();
		Check.isBlank(questionId, "questionId不能为空");
		return examRecordQuestionsService.getQuestionContent(user.getUserId(),questionId);
	}
	
	/**
	 * 考生作答
	 * @param examQuestionInfos
	 */
	@ApiOperation(value = "考试过程中-考生作答:更新试题作答信息(包括提交试题答案，更新是否标记)")
	@PostMapping("/submitQuestionAnswer")
	public void submitQuestionAnswer(@RequestBody List<ExamStudentQuestionInfo> examQuestionInfos){
		if(log.isDebugEnabled()) {
			String strJosn=JsonUtil.toJson(examQuestionInfos);
			log.debug("ExamQuestionController--submitQuestionAnswer参数信息:"+strJosn);
		}
		User user = getAccessUser();
		if(examQuestionInfos!=null && examQuestionInfos.size()>0){
			for(ExamStudentQuestionInfo examStudentQuestionInfo:examQuestionInfos){
				if(examStudentQuestionInfo.getOrder() == null){
					throw new StatusException("examQuestionController-submitQuestionAnswer", "illegal params");
				}
			}
			examRecordQuestionsService.submitQuestionAnswer(user.getUserId(),examQuestionInfos);
		}
	}
}
