package cn.com.qmth.examcloud.core.oe.student.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.common.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamProperties;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamType;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.student.bean.OfflineExamCourseInfo;
import cn.com.qmth.examcloud.core.oe.student.service.OfflineExamService;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.util.List;


/**
 * 
 * @author  	chenken
 * @date    	2018年9月5日 下午3:33:26
 * @company 	QMTH
 * @description 离线考试Controller
 */
@Api(tags = "离线考试控制")
@RestController
@RequestMapping("${app.api.oe.student}/offlineExam")
public class OfflineExamController extends ControllerSupport{

	@Autowired
	private OfflineExamService offlineExamService;

	@Autowired
	private ExamRecordDataRepo examRecordDataRepo;
	/**
	 * 答案文件最大限制
	 * 单位:M
	 */
	private static int answerMaxsize = 30;
	
	
	public static final String TEMP_FILE_EXP = "offlineExam/";
	
	
	/**
	 * 获取离线考试列表
	 * @return
	 */
	@ApiOperation(value = "获取离线考试列表")
	@GetMapping("/getOfflineCourse")
	public List<OfflineExamCourseInfo> getOfflineCourse(){
		User user = getAccessUser();
		List<OfflineExamCourseInfo> offlineExamCourseInfos = offlineExamService.getOfflineCourse(user.getUserId());
		return offlineExamCourseInfos;
	}
	
	/**
	 * 开始考试
	 * @param examStudentId
	 */
	@ApiOperation(value = "离线考试:开始考试")
	@GetMapping("/startOfflineExam")
	public void startOfflineExam(@RequestParam long examStudentId){
		Check.isNull(examStudentId, "examStudentId不能为空");
		offlineExamService.startOfflineExam(examStudentId);
	}
	
	/**
	 * 交卷
	 */
	@ApiOperation(value = "离线考试:交卷")
	@PostMapping("/submitPaper")
	public void submitPaper(@RequestParam(value = "file") MultipartFile file,
							@RequestParam long examRecordDataId)  throws Exception{
		Check.isNull(file, "file不能为空");
		Check.isNull(examRecordDataId, "examRecordDataId不能为空");
		String fileName = file.getOriginalFilename();
		int index = fileName.lastIndexOf(".");
		String fileSuffix = fileName.substring(index+1, fileName.length()).toUpperCase();
		if(!"PDF".equals(fileSuffix) && !"ZIP".equals(fileSuffix)){
			throw new StatusException("OfflineExamController-submitPaper-001","文件格式不正确"); 
		}
		
		ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo,examRecordDataId,ExamRecordDataEntity.class);
		if(examRecordData.getExamType() != ExamType.OFFLINE){
			throw new StatusException("OfflineExamController-submitPaper-002","非离线考试"); 
		}
		String offlineUploadFileType = CacheHelper.getExamOrgProperty(examRecordData.getExamId(),
				examRecordData.getOrgId(),ExamProperties.OFFLINE_UPLOAD_FILE_TYPE.name()).getValue();
		if(StringUtils.isBlank(offlineUploadFileType) || "[]".equals(offlineUploadFileType)){
			throw new StatusException("OfflineExamController-submitPaper-003","当前考试设置不允许上传附件"); 
		}
		if(offlineUploadFileType.indexOf(fileSuffix)<0){
			throw new StatusException("OfflineExamController-submitPaper-004","当前考试允许上传文件格式为:" + offlineUploadFileType); 
		}
		//判断文件大小
		long fileSize = file.getSize();
		if(fileSize > answerMaxsize * 1048576){
			throw new StatusException("OfflineExamController-submitPaper-005","文件大小不能超过"+answerMaxsize+"M"); 
		}
		offlineExamService.submitPaper(examRecordDataId,getUploadFile(file));
	}
	
	private File getUploadFile(MultipartFile file){
        //建临时文件夹
		File dirFile = new File(TEMP_FILE_EXP);
		if(!dirFile.exists()){
			dirFile.mkdirs();
		}
        String fileName = file.getOriginalFilename();
        File tempFile = new File(TEMP_FILE_EXP + fileName);
        OutputStream os = null;
        try {
			os = new FileOutputStream(tempFile);
			IOUtils.copyLarge(file.getInputStream(), os);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}finally{
			IOUtils.closeQuietly(os);
		}
        return tempFile;
    }
}
