package cn.com.qmth.examcloud.core.oe.student.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.core.oe.common.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.student.bean.PracticeCourseInfo;
import cn.com.qmth.examcloud.core.oe.student.bean.PracticeDetailInfo;
import cn.com.qmth.examcloud.core.oe.student.bean.PracticeRecordInfo;
import cn.com.qmth.examcloud.core.oe.student.service.PracticeService;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 
 * @author  	chenken
 * @date    	2018年9月7日 上午11:12:28
 * @company 	QMTH
 * @description PracticeController.java
 */
@Api(tags = "练习相关接口")
@RestController
@RequestMapping("${app.api.oe.student}/practice")
public class PracticeController extends ControllerSupport{

	@Autowired
	private PracticeService practiceService;
	
	/**
	 * 练习课程列表
	 * @param examId
	 * @param studentId
	 * @return
	 */
	@ApiOperation(value = "练习课程列表")
	@GetMapping("/queryPracticeCourseList")
	public List<PracticeCourseInfo> queryPracticeCourseList(@RequestParam Long examId){
		Check.isNull(examId, "examId不能为空");
		User user = getAccessUser();
		return practiceService.queryPracticeCourseList(examId, user.getUserId());
	}
	
	/**
	 * 课程练习记录详情
	 * @param examStudentId
	 * @return
	 */
	@ApiOperation(value = "课程练习记录详情")
	@GetMapping("/queryPracticeRecordList")
	public List<PracticeRecordInfo> queryPracticeRecordList(@RequestParam Long examStudentId){
		Check.isNull(examStudentId, "examStudentId不能为空");
		return practiceService.queryPracticeRecordList(examStudentId);
	}
	
	/**
	 * 单次练习答题情况统计
	 * @return
	 */
	@ApiOperation(value = "单次练习答题情况统计")
	@GetMapping("/getPracticeDetailInfo")
	public PracticeDetailInfo getPracticeDetailInfo(@RequestParam Long examRecordDataId){
		Check.isNull(examRecordDataId, "examRecordDataId不能为空");
		return practiceService.getPracticeDetailInfo(examRecordDataId);
	}
	
}
