package cn.com.qmth.examcloud.core.oe.student.provider;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.StudentBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentResp;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamingRecordEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.HandInExamRecordEntity;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.student.api.OeStudentExamRecordCloudService;
import cn.com.qmth.examcloud.core.oe.student.api.bean.ExamingRecordBean;
import cn.com.qmth.examcloud.core.oe.student.api.bean.HandInExamRecordBean;
import cn.com.qmth.examcloud.core.oe.student.api.request.*;
import cn.com.qmth.examcloud.core.oe.student.api.response.*;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamSessionInfo;
import cn.com.qmth.examcloud.core.oe.student.service.ExamControlService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamSessionInfoService;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import com.mysql.cj.util.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

@Api(tags = "清理考试记录")
@RestController
@RequestMapping("${$rmp.cloud.oe.student}" + "examRecord")
public class ExamRecordCloudServiceProvider extends ControllerSupport implements OeStudentExamRecordCloudService {

    /**
     *
     */
    private static final long serialVersionUID = 2873433904730318971L;
    @Autowired
    ExamSessionInfoService examSessionInfoService;
    @Autowired
    StudentCloudService studentCloudService;
    @Autowired
    ExamRecordDataRepo examRecordDataRepo;
    @Autowired
    ExamRecordDataService examRecordDataService;
    @Autowired
    private ExamControlService examControlService;

    @Naked
    @PostMapping("/cleanExamingRecord")
    @Override
    public void cleanExamingRecord(@RequestBody CleanExamingRecordReq req) {
        ExamingRecordEntity examingRecordEntity = new ExamingRecordEntity();
        examingRecordEntity.setExamRecordDataId(req.getExamRecordDataId());
        examingRecordEntity.setId(req.getId());
        examingRecordEntity.setStudentId(req.getStudentId());
        examControlService.cleanExamingRecord(examingRecordEntity);
    }

    @Naked
    @PostMapping("/cleanHandInExamRecord")
    @Override
    public void cleanHandInExamRecord(@RequestBody CleanHandInExamRecordReq req) {
        HandInExamRecordEntity handInExamRecordEntity = new HandInExamRecordEntity();
        handInExamRecordEntity.setExamRecordDataId(req.getExamRecordDataId());
        handInExamRecordEntity.setId(req.getId());
        handInExamRecordEntity.setStudentId(req.getStudentId());
        examControlService.cleanHandInExamRecord(handInExamRecordEntity);
    }

    @Naked
    @ApiOperation(value = "获取二维码")
    @Override
    @PostMapping("/getQrCode")
    public GetQrCodeResp getQrCode(@RequestBody GetQrCodeReq req) {
        log.info("开始调用获取二维码方法---getQrCode");
        cn.com.qmth.examcloud.core.oe.student.bean.GetQrCodeReq beanReq = new cn.com.qmth.examcloud.core.oe.student.bean.GetQrCodeReq();
        beanReq.setExamRecordDataId(req.getExamRecordDataId());
        beanReq.setExamStudentId(req.getExamStudentId());
        beanReq.setOrder(req.getOrder());
        beanReq.setTransferFileType(req.getTransferFileType());
        beanReq.setTestEnv(req.isTestEnv());
        log.info("参数准备完毕---getQrCode,req.getExamRecordDataId()=" + req.getExamRecordDataId());
        log.info("参数准备完毕---getQrCode,req.getExamStudentId()=" + req.getExamStudentId());
        log.info("参数准备完毕---getQrCode,req.getOrder()=" + req.getOrder());
        log.info("参数准备完毕---getQrCode,req.getTokenKey()=" + req.getTokenKey());
        String qrCode = examControlService.getQrCode(beanReq, req.getTokenKey());
        GetQrCodeResp resp = new GetQrCodeResp();
        resp.setQrCode(qrCode);
        return resp;
    }

    @Naked
    @ApiOperation(value = "获取考试会话")
    @Override
    @PostMapping("/getExamSessionInfo")
    public GetExamSessionInfoResp getExamSessionInfo(@RequestBody GetExamSessionInfoReq req) {

        ExamSessionInfo examSessionInfo = examSessionInfoService.getExamSessionInfo(req.getStudentId());
        if (null == examSessionInfo) {
            throw new StatusException("100001", "考试已结束");
        }
        GetExamSessionInfoResp resp = new GetExamSessionInfoResp();
        resp.setExamStudentId(examSessionInfo.getExamStudentId());
        return resp;
    }

    @Naked
    @ApiOperation(value = "获取学生是否有正在进行的网考信息")
    @Override
    @PostMapping("/getStudentOnLineExamInfo")
    public GetStudentOnlineExamInfoResp getStudentOnLineExamInfo(@RequestBody GetStudentOnlineExamInfoReq req) {
        if (null == req.getRootOrgId()) {
            throw new StatusException("101001", "组织机构id参数不允许为空");
        }
        if (StringUtils.isNullOrEmpty(req.getStudentCode()) && StringUtils.isNullOrEmpty(req.getIdentityNumber())
                && StringUtils.isNullOrEmpty(req.getPhone())) {
            throw new StatusException("101002", "学生学号、身份证号或手机号参数至少有一个不为空");
        }
        GetStudentReq getStudentReq = new GetStudentReq();
        getStudentReq.setRootOrgId(req.getRootOrgId());
        if (!StringUtils.isNullOrEmpty(req.getStudentCode())) {
            getStudentReq.setStudentCode(req.getStudentCode());
        }
        if (!StringUtils.isNullOrEmpty(req.getIdentityNumber())) {
            getStudentReq.setIdentityNumber(req.getIdentityNumber());
        }
        if (!StringUtils.isNullOrEmpty(req.getPhone())) {
            getStudentReq.setSecurityPhone(req.getPhone());
        }
        //此方法内部已作空处理，不需要再判断非空
        GetStudentResp studentResp = studentCloudService.getStudent(getStudentReq);
        StudentBean studentInfo = studentResp.getStudentInfo();
        Long studentId = studentInfo.getId();
        ExamRecordDataEntity examRecordData = examRecordDataRepo.findOnlineExamingRecordByStudentId(studentId);
        GetStudentOnlineExamInfoResp resp = new GetStudentOnlineExamInfoResp();
        if (null != examRecordData && null != examRecordData.getId()) {
            resp.setExistExamingRecord(true);
        } else {
            resp.setExistExamingRecord(false);
        }
        return resp;
    }

    @Naked
    @PostMapping("/getExamingRecords")
    @Override
    public GetExamingRecordResp getExamingRecords(@RequestBody GetExamingRecordReq req) {
        List<ExamingRecordEntity> examingRecords = examRecordDataService.getLimitExamingRecords(req.getStartId(), req.getLimit());
        if (null == examingRecords || examingRecords.isEmpty()) {
            return new GetExamingRecordResp(req.getStartId(), null);
        }

        List<ExamingRecordBean> examingRecordBeanList = new ArrayList<>();
        for (ExamingRecordEntity entity : examingRecords) {
            ExamingRecordBean bean = new ExamingRecordBean();
            bean.setExamRecordDataId(entity.getExamRecordDataId());
            bean.setId(entity.getId());
            bean.setStudentId(entity.getStudentId());
            examingRecordBeanList.add(bean);
        }
        Long nextId = examingRecords.get(examingRecords.size() - 1).getId() + 1;
        return new GetExamingRecordResp(nextId, examingRecordBeanList);
    }


    @Naked
    @PostMapping("/getHandInExamRecords")
    @Override
    public GetHandInExamRecordResp getHandInExamRecords(@RequestBody GetHandInExamRecordReq req) {
        List<HandInExamRecordEntity> handInExamRecords = examRecordDataService.getLimitHandInExamRecords(req.getStartId(), req.getLimit());
        if (null == handInExamRecords || handInExamRecords.isEmpty()) {
            return new GetHandInExamRecordResp(req.getStartId(), null);
        }

        List<HandInExamRecordBean> handInExamRecordBeanList = new ArrayList<>();
        for (HandInExamRecordEntity entity : handInExamRecords) {
            HandInExamRecordBean bean = new HandInExamRecordBean();
            bean.setExamRecordDataId(entity.getExamRecordDataId());
            bean.setId(entity.getId());
            bean.setStudentId(entity.getStudentId());
            handInExamRecordBeanList.add(bean);
        }
        Long nextId = handInExamRecords.get(handInExamRecords.size() - 1).getId() + 1;
        return new GetHandInExamRecordResp(nextId, handInExamRecordBeanList);
    }

    @Naked
    @ApiOperation(value = "清理考试已结束的文件作答记录")
    @Override
    @PostMapping("/cleanTempFileAnswers")
    public void cleanTempFileAnswers() {
        examControlService.cleanTempFileAnswers();
    }
}
