package cn.com.qmth.examcloud.core.oe.student.service;


import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamFileAnswerTempEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamingRecordEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.HandInExamRecordEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.HandInExamType;
import cn.com.qmth.examcloud.core.oe.student.bean.*;

import javax.validation.Valid;
import java.util.List;

/**
 * @author chenken
 * @date 2018年8月13日 下午2:09:38
 * @company QMTH
 * @description 在线考试控制服务接口
 */
public interface ExamControlService {

    /**
     * 开始考试
     *
     * @param examStudentId
     * @param user
     */
    public StartExamInfo startExam(Long examStudentId, User user);

    /**
     * 考试心跳
     *
     * @param
     */
    public long examHeartbeat(User user);

    /**
     * 断点续考:检查正在进行中的考试
     *
     * @param studentId
     */
    public CheckExamInProgressInfo checkExamInProgress(Long studentId);

    /**
     * 获取考试记录信息
     *
     * @param examRecordDataId
     */
    public EndExamInfo getEndExamInfo(Long examRecordDataId);

    /**
     * 照片失败扫描处理
     */
    public void handleByExamCaptureQueueFailedDispose(Long examRecordDataId);

    /**
     * 获取上传考试音频所需的又拍云签名
     *
     * @param req
     * @return
     */
    UpyunSignatureInfo getUpyunSignature(GetUpyunSignatureReq req);

    /**
     * 发送消息到websocket
     *
     * @param examRecordDataId 考试记录id
     * @param order            题序号
     * @param fileUrl          文件路径
     * @param transferFileType 传输文件类型
     * @throws Exception
     */
    public void sendFileAnswerToWebSocket(Long examRecordDataId, Integer order,
                                          String fileUrl, String transferFileType, Long userId) throws Exception;

    public String getQrCode(GetQrCodeReq req, String key);

    /**
     * 校验二维码
     *
     * @param param
     * @return
     */
    public CheckQrCodeInfo checkQrCode(String param);

    /**
     * 成功上传音频结果推送（微信小程序调用）
     *
     * @param req
     */
    public ExamFileAnswerTempEntity saveUploadedFile(SaveUploadedFileReq req, User user);

    /**
     * 查询上传音频结果推送状态（微信小程序调用）
     *
     * @param req
     */
    public String getUploadedFileAcknowledgeStatus(GetUploadedFileAcknowledgeStatusReq req);

    public void saveUploadedFileAcknowledgeStatus(SaveUploadedFileAcknowledgeStatusReq req);

    /**
     * 通过websocket发送二维码扫描信息
     *
     * @param examRecordDataId
     * @param clientId
     * @throws Exception
     */
    public void sendScanQrCodeToWebSocket(String clientId, Long examRecordDataId, Integer order) throws Exception;

    /**
     * 获取已上传的文件答案列表（微信小程序调用）
     *
     * @param req
     * @return
     */
    public List<UploadedFileAnswerInfo> getUploadedFileAnswerList(@Valid GetUploadedFileAnswerListReq req);

    String getQrCode(GetQrCodeReq req, User user);

    void deleteExamFileAnswerTemp(SaveUploadedFileReq req);

    /**
     * 计算考试已用时间(毫秒)
     *
     * @param examSessionInfo 考试会话对象
     * @return
     */
    Long calculateExamUsedMilliseconds(ExamSessionInfo examSessionInfo);

    /**
     * 交卷
     *
     * @param examRecordDataId 考试记录id
     * @param handInExamType   交卷类型
     */
    void handInExam(Long examRecordDataId, HandInExamType handInExamType);

    /**
     * 交卷的后续处理
     *
     * @param examRecordDataId 考试记录id
     * @param handInExamType   交卷类型
     * @param studentId        学生id
     * @return boolean
     */
    boolean processAfterHandInExam(Long examRecordDataId, Long studentId, HandInExamType handInExamType);

    /**
     * 清理考试已结束的文件作答记录
     */
    void cleanTempFileAnswers();

    /**
     * 清理考试中的考试记录
     *
     * @param examingRecordEntity
     */
    void cleanExamingRecord(ExamingRecordEntity examingRecordEntity);

    /**
     * 清理已交卷的考试记录
     *
     * @param handInExamRecordEntity
     */
    void cleanHandInExamRecord(HandInExamRecordEntity handInExamRecordEntity);

}
