package cn.com.qmth.examcloud.core.oe.student.service;

import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamingRecordEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.HandInExamRecordEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamType;
import cn.com.qmth.examcloud.core.oe.student.bean.CalculateFaceCheckResultInfo;
import cn.com.qmth.examcloud.examwork.api.bean.ExamBean;

import java.util.List;

/**
 * @author chenken
 * @date 2018/8/15 11:16
 * @company QMTH
 * @description 考试记录数据服务接口
 */
public interface ExamRecordDataService {

    /**
     * 创建ExamRecordDataEntity
     * @param examedTimes 已考次数
     * @param canExamTimes 可考次数
     * @param isReExamine 本次考试是否为重考
     * @param isFullyObjetive 是否全客观题
     * @return
     */
    ExamRecordDataEntity createExamRecordData(ExamStudentEntity examStudent,
                                              ExamBean examBean,
                                              CourseBean courseBean,
                                              String basePaperId,
                                              String paperStructId, Integer examedTimes, Long canExamTimes,
                                              Boolean isReExamine, Boolean isFullyObjetive);

    List<ExamRecordDataEntity> findByStatusAndExamTypeIn(ExamRecordStatus examRecordStatus, List<String> examTypeList);

    /**
     * 根据考生ID查询所有有效考试记录
     *
     * @param examStudentId
     * @return
     */
    List<ExamRecordDataEntity> findValidExamRecordDataByExamStudentId(Long examStudentId);

    /**
     * 计算人脸检测结果
     *
     * @param examRecordDataEntity
     * @return
     */
    CalculateFaceCheckResultInfo calculateFaceCheckResult(ExamRecordDataEntity examRecordDataEntity);

    /**
     * 创建进行中的考试记录
     *
     * @param examRecordDataId
     * @param studentId
     */
    void createExamingRecord(Long examRecordDataId, Long studentId, ExamType examType);

    /**
     * 创建已交卷的考试记录
     *
     * @param examRecordDataId
     * @param studentId
     */
    void createHandInExamRecord(Long examRecordDataId, Long studentId);

    /**
     * 删除进行中考试记录id
     *
     * @param examRecordDataId
     */
    void deleteExamingRecord(Long examRecordDataId);

    void deleteHandInExamRecord(Long examRecordDataId);

    /**
     * 获取指定数量的考试中的数据
     *
     * @param startId
     * @param limit
     * @return
     */
    List<ExamingRecordEntity> getLimitExamingRecords(Long startId, int limit);

    /**
     * 获取指定数量的已交卷的考试数据
     *
     * @param startId
     * @param limit
     * @return
     */
    List<HandInExamRecordEntity> getLimitHandInExamRecords(Long startId, int limit);

    /**
     * 计算活体检测结果
     *
     * @param examRecordDataEntity
     */
    ExamRecordDataEntity calculateFaceLivenessVerifyResult(ExamRecordDataEntity examRecordDataEntity);

    /**
     * 创建离线考试记录
     *
     * @param examRecord
     * @param fullyObjective
     * @return
     */
    ExamRecordDataEntity createOfflineExamRecordData(ExamStudentEntity examStudent,
                                                     ExamBean examBean,
                                                     CourseBean courseBean,
                                                     String basePaperId,
                                                     String paperStructId, Boolean fullyObjective);

    /**
     * 违纪自动审核
     *
     * @return
     */
    ExamRecordDataEntity examRecordAutoAudit(Boolean isNoPhotoAndIllegality, ExamRecordDataEntity examRecordData);

    /**
     * 考试记录是否已结束
     *
     * @param examRecordData 考试记录
     * @return boolean
     */
    boolean isExamRecordEnded(ExamRecordDataEntity examRecordData);

    /**
     * 考试记录是否已结束
     *
     * @param examRecordDataId 考试记录id
     * @return boolean
     */
    boolean isExamRecordEnded(Long examRecordDataId);

}
