package cn.com.qmth.examcloud.core.oe.student.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.core.oe.common.base.utils.CommonUtil;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamAuditEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamFaceLivenessVerifyEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.AuditStatus;
import cn.com.qmth.examcloud.core.oe.common.enums.DisciplineType;
import cn.com.qmth.examcloud.core.oe.common.enums.FaceVerifyResult;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamAuditRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamFaceLivenessVerifyRepo;
import cn.com.qmth.examcloud.core.oe.student.service.ExamAuditService;

/**
 * 
 * @author  	chenken
 * @date    	2018年9月5日 下午3:25:06
 * @company 	QMTH
 * @description 学生端-考试记录审核服务实现
 */
@Service("examAuditService")
public class ExamAuditServiceImpl implements ExamAuditService {

	//private static final Logger log = LoggerFactory.getLogger(ExamAuditServiceImpl.class);
	
	/**
	 * 活体检测失败自动审核
	 */
	private static final String FACE_LIVENESS_VERIFY_AUTO_AUDIT = "FACE_LIVENESS_VERIFY_AUTO_AUDIT";
	
	/**
	 * 无照片自动审核
	 */
	private static final String NOPHOTO_VERIFY_AUTO_AUDIT = "NOPHOTO_VERIFY_AUTO_AUDIT";
	
	/**
	 * 系统审核
	 */
	private static final String AUDIT_USER_NAME = "SYSTEM";
	
	@Autowired
	private ExamFaceLivenessVerifyRepo examFaceLivenessVerifyRepo;
	
	@Autowired
	private ExamAuditRepo examAuditRepo;
	
	@Override
	public void saveExamAuditByFaceVerifyFailed(Long examRecordDataId) {
		ExamAuditEntity examAudit = examAuditRepo.findByExamRecordDataId(examRecordDataId);
		if(examAudit != null){
			return;
		}
		ExamAuditEntity examAuditEntity = new ExamAuditEntity();
		examAuditEntity.setExamRecordDataId(examRecordDataId);
		examAuditEntity.setDisciplineType(DisciplineType.ACTION_FAILURE);
		examAuditEntity.setUserId(FACE_LIVENESS_VERIFY_AUTO_AUDIT);//活体检测自动审核
		examAuditEntity.setAuditUserName(AUDIT_USER_NAME);
		
		List<ExamFaceLivenessVerifyEntity> faceVerifies = examFaceLivenessVerifyRepo.findByExamRecordDataIdOrderById(examRecordDataId);
		if(faceVerifies == null || faceVerifies.size() == 0){
			examAuditEntity.setDisciplineDetail("未进行人脸动作检测");
		}else{
			StringBuffer sbBuffer = new StringBuffer("");
			for(ExamFaceLivenessVerifyEntity faceVerify:faceVerifies){
				String startTime = CommonUtil.getDateStrWithSecond(faceVerify.getStartTime());
				sbBuffer.append(startTime+":"+FaceVerifyResult.getDescByName(faceVerify.getVerifyResult()));
				sbBuffer.append("&&");
			}
			String disciplineDetail = sbBuffer.toString();
			if(disciplineDetail.length()>500){
				disciplineDetail = disciplineDetail.substring(0,500);
			}
			examAuditEntity.setDisciplineDetail(disciplineDetail);
		}
		examAuditEntity.setStatus(AuditStatus.UN_PASS);
		examAuditRepo.save(examAuditEntity);
	}

	@Override
	public void saveExamAuditByNoPhoto(Long examRecordDataId) {
		ExamAuditEntity examAudit = examAuditRepo.findByExamRecordDataId(examRecordDataId);
		if(examAudit != null){
			return;
		}
		ExamAuditEntity examAuditEntity = new ExamAuditEntity();
		examAuditEntity.setExamRecordDataId(examRecordDataId);
		examAuditEntity.setDisciplineType(DisciplineType.OTHER);
		examAuditEntity.setDisciplineDetail("抓拍照片数量为0");
		examAuditEntity.setUserId(NOPHOTO_VERIFY_AUTO_AUDIT);//抓拍照片为0自动审核
		examAuditEntity.setAuditUserName(AUDIT_USER_NAME);
		examAuditEntity.setStatus(AuditStatus.UN_PASS);
		examAuditRepo.save(examAuditEntity);
	}
	
}
