package cn.com.qmth.examcloud.core.oe.student.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.oe.common.base.Constants;
import cn.com.qmth.examcloud.core.oe.common.base.jpa.Searcher;
import cn.com.qmth.examcloud.core.oe.common.base.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.oe.common.base.utils.CommonUtil;
import cn.com.qmth.examcloud.core.oe.common.entity.*;
import cn.com.qmth.examcloud.core.oe.common.enums.*;
import cn.com.qmth.examcloud.core.oe.common.repository.*;
import cn.com.qmth.examcloud.core.oe.common.service.ExamScoreObtainQueueService;
import cn.com.qmth.examcloud.core.oe.common.service.ExamScorePushQueueService;
import cn.com.qmth.examcloud.core.oe.common.service.GainBaseDataService;
import cn.com.qmth.examcloud.core.oe.student.bean.CalculateFaceCheckResultInfo;
import cn.com.qmth.examcloud.core.oe.student.face.api.ExamCaptureQueueCloudService;
import cn.com.qmth.examcloud.core.oe.student.service.*;
import cn.com.qmth.examcloud.examwork.api.bean.ExamBean;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author chenken
 * @date 2018/8/15 11:16
 * @company QMTH
 * @description 考试记录数据服务实现
 */
@Service("examRecordDataService")
public class ExamRecordDataServiceImpl implements ExamRecordDataService {

    @Autowired
    private ExamSessionInfoService examSessionInfoService;

    @Autowired
    private ExamAuditService examAuditService;

    @Autowired
    private ExamScoreService examScoreService;

    @Autowired
    private ExamRecordQuestionsService examRecordQuestionsService;

    @Autowired
    private ExamRecordForMarkingService examRecordForMarkingService;

    @Autowired
    private ExamCaptureQueueCloudService examCaptureQueueCloudService;

    @Autowired
    private GainBaseDataService gainBaseDataService;

    @Autowired
    private ExamScorePushQueueService examScorePushQueueService;

    @Autowired
    private ExamScoreObtainQueueService examScoreObtainQueueService;

    @Autowired
    private ExamFaceLivenessVerifyRepo examFaceLivenessVerifyRepo;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamCaptureRepo examCaptureRepo;

    @Autowired
    private ExamingRecordRepo examingRecordRepo;
    @Autowired
    private HandInExamRecordRepo handInExamRecordRepo;

    private static final Logger log = LoggerFactory.getLogger(ExamRecordDataServiceImpl.class);

    @Override
    public ExamRecordDataEntity createExamRecordData(ExamStudentEntity examStudent,
                                                     ExamBean examBean,
                                                     CourseBean courseBean,
                                                     String basePaperId,
                                                     String paperStructId, Integer examedTimes, Long canExamTimes,
                                                     Boolean isReExamine, Boolean isFullyObjetive) {
        ExamRecordDataEntity examRecordData = new ExamRecordDataEntity();
        examRecordData.setExamId(examBean.getId());
        examRecordData.setExamType(ExamType.strToEnum(examBean.getExamType()));

        examRecordData.setExamStudentId(examStudent.getExamStudentId());
        examRecordData.setStudentId(examStudent.getStudentId());
        examRecordData.setStudentCode(examStudent.getStudentCode());
        examRecordData.setStudentName(examStudent.getStudentName());
        examRecordData.setIdentityNumber(examStudent.getIdentityNumber());
        examRecordData.setOrgId(examStudent.getOrgId());
        examRecordData.setRootOrgId(examStudent.getRootOrgId());

        examRecordData.setCourseId(courseBean.getId());
        examRecordData.setCourseLevel(examStudent.getCourseLevel());
        examRecordData.setBasePaperId(basePaperId);

        examRecordData.setPaperType(examStudent.getPaperType());
        examRecordData.setPaperStructId(paperStructId);

        examRecordData.setInfoCollector(examStudent.getInfoCollector());

//        examRecordData.setExamRecordDataId(examRecord.getId());
        examRecordData.setStartTime(new Date());
        examRecordData.setExamRecordStatus(ExamRecordStatus.EXAM_ING);
        //已考的考试次数
        int finalExamedTimes = examedTimes == null ? 0 : examedTimes.intValue();
        //考试机会
        int finalCanExamTimes = canExamTimes == null ? 0 : canExamTimes.intValue();
        boolean finalIsReExamine = isReExamine == null ? false : isReExamine;
        //如果是重考，则考试次数加1
        if (finalIsReExamine && finalExamedTimes >= finalCanExamTimes) {
            finalExamedTimes = finalCanExamTimes + 1;
        }
        examRecordData.setExamOrder(finalExamedTimes);
        examRecordData.setIsAllObjectivePaper(isFullyObjetive);
        examRecordData.setIsWarn(false);
        examRecordData.setIsAudit(false);
        examRecordData.setIsIllegality(false);
        examRecordData.setIsContinued(false);
        examRecordData.setIsReexamine(false);
        examRecordData.setContinuedCount(0);
        examRecordData.setFaceSuccessCount(0);
        examRecordData.setFaceTotalCount(0);
        examRecordData.setFaceFailedCount(0);
        examRecordData.setFaceStrangerCount(0);
        if (finalIsReExamine && finalCanExamTimes <= finalExamedTimes) {
            examRecordData.setIsReexamine(true);
        }
        return examRecordDataRepo.save(examRecordData);
    }

    @Override
    public void createExamingRecord(Long examRecordDataId, Long studentId, ExamType examType) {
        ExamingRecordEntity examingRecord = new ExamingRecordEntity();
        examingRecord.setStudentId(studentId);
        examingRecord.setId(examRecordDataId);
        examingRecord.setExamRecordDataId(examRecordDataId);
        examingRecord.setExamType(examType);
        examingRecord.setIsContinued(false);
        examingRecord.setContinuedCount(0);
        examingRecordRepo.save(examingRecord);
    }

    @Override
    public void createHandInExamRecord(Long examRecordDataId, Long studentId) {
        HandInExamRecordEntity handInExamRecord = new HandInExamRecordEntity();
        handInExamRecord.setStudentId(studentId);
        handInExamRecord.setId(examRecordDataId);
        handInExamRecord.setExamRecordDataId(examRecordDataId);
        handInExamRecordRepo.save(handInExamRecord);
    }

    @Override
    public void deleteExamingRecord(Long examRecordDataId) {
        examingRecordRepo.deleteById(examRecordDataId);
    }

    @Override
    public void deleteHandInExamRecord(Long examRecordDataId) {
        handInExamRecordRepo.deleteById(examRecordDataId);
    }

    @Override
    public List<ExamingRecordEntity> getLimitExamingRecords(Long startId, int limit) {
        return examingRecordRepo.getLimitExamingRecords(startId, limit);
    }

    @Override
    public List<HandInExamRecordEntity> getLimitHandInExamRecords(Long startId, int limit) {
        return handInExamRecordRepo.getLimitHandInExamRecords(startId, limit);
    }

    /**
     * 计算活体检测结果
     *
     * @return
     */
    @Override
    public ExamRecordDataEntity calculateFaceLivenessVerifyResult(ExamRecordDataEntity examRecordDataEntity) {
        if (examRecordDataEntity.getFaceVerifyResult() != null) {
            if (examRecordDataEntity.getFaceVerifyResult() == IsSuccess.FAILED) {
                List<ExamFaceLivenessVerifyEntity> faceVerifies = examFaceLivenessVerifyRepo.findByExamRecordDataIdOrderById(examRecordDataEntity.getId());
                setExamRecordWithFaceLivenessVerifyFailed(examRecordDataEntity, faceVerifies);
            }
        } else {
            Long examId = examRecordDataEntity.getExamId();
            String isFaceVerify = CacheHelper.getExamOrgProperty(examId, examRecordDataEntity.getOrgId(),
                    ExamProperties.IS_FACE_VERIFY.name()).getValue();

            //是否进行活体检测
            if (isFaceVerify != null && Constants.isTrue.equals(isFaceVerify)) {
                List<ExamFaceLivenessVerifyEntity> faceVerifies = examFaceLivenessVerifyRepo.findByExamRecordDataIdOrderById(examRecordDataEntity.getId());
                if (faceVerifies != null && faceVerifies.size() > 0) {
                    ExamFaceLivenessVerifyEntity latestFaceVerify = faceVerifies.get(faceVerifies.size() - 1);
                    if (latestFaceVerify != null && latestFaceVerify.getVerifyResult() != null && latestFaceVerify.getVerifyResult() == FaceVerifyResult.VERIFY_SUCCESS) {
                        examRecordDataEntity.setFaceVerifyResult(IsSuccess.SUCCESS);
                    } else {
                        setExamRecordWithFaceLivenessVerifyFailed(examRecordDataEntity, faceVerifies);
                    }
                } else {
                    setExamRecordWithFaceLivenessVerifyFailed(examRecordDataEntity, faceVerifies);
                }
            }
        }
        return examRecordDataEntity;
    }

    /**
     * 活体检测失败，设置考试记录属性
     *
     * @param examRecordDataEntity
     * @param faceVerifies
     */
    private void setExamRecordWithFaceLivenessVerifyFailed(ExamRecordDataEntity examRecordDataEntity, List<ExamFaceLivenessVerifyEntity> faceVerifies) {
        examRecordDataEntity.setIsIllegality(true);//判定为违纪
        examRecordDataEntity.setIsWarn(true);    //有警告
        examRecordDataEntity.setFaceVerifyResult(IsSuccess.FAILED);//活体检测失败
    }

    /**
     * 计算人脸检测结果
     * 相片数=0,系统判断为违纪,自动审核
     * 考试记录为异常逻辑(进入待审)：
     * 1.陌生人次数>0
     * 2.face++阈值 = 0 && 百度真实性阈值 > 0
     * 真实性百分比<百度真实性阈值
     * 3.face++阈值 > 0 && 百度真实性阈值 = 0
     * face++成功率<face++阈值
     * 4.face++阈值 > 0 && 百度真实性阈值 > 0
     * face++成功率<face++阈值 ||
     * 真实性百分比<百度真实性阈值
     *
     * @param examRecordData
     * @return
     */
    @Override
    public CalculateFaceCheckResultInfo calculateFaceCheckResult(ExamRecordDataEntity examRecordData) {
        CalculateFaceCheckResultInfo calculateFaceCheckResultInfo = new CalculateFaceCheckResultInfo();
        Long examId = examRecordData.getExamId();
        String isFaceEnable = CacheHelper.getExamOrgProperty(examId, examRecordData.getOrgId(),
                ExamProperties.IS_FACE_ENABLE.name()).getValue();
        //未开启人脸检测
        if (!Constants.isTrue.equals(isFaceEnable)) {
            examRecordData.setIsWarn(false);
            calculateFaceCheckResultInfo.setExamRecordData(examRecordData);
            return calculateFaceCheckResultInfo;
        }
        List<ExamCaptureEntity> examCaptureList = examCaptureRepo.findByExamRecordDataId(examRecordData.getId());
        if (examCaptureList == null || examCaptureList.size() == 0) {
            examRecordData.setIsWarn(true);//有异常
            examRecordData.setIsIllegality(true);//违纪
            calculateFaceCheckResultInfo.setExamRecordData(examRecordData);
            calculateFaceCheckResultInfo.setIsNoPhotoAndIllegality(true);
            return calculateFaceCheckResultInfo;
        }
        //计算
        examRecordData = computeFaceCheckData(examRecordData, examCaptureList);
        //陌生人个数>0
        if (examRecordData.getFaceStrangerCount() > 0) {
            examRecordData.setIsWarn(true);
            calculateFaceCheckResultInfo.setExamRecordData(examRecordData);
            return calculateFaceCheckResultInfo;
        }
        //人脸识别阀值
        String warnThresholdStr = CacheHelper.getExamOrgProperty(examId, examRecordData.getOrgId(),
                ExamProperties.WARN_THRESHOLD.name()).getValue();
        if (CommonUtil.isBlank(warnThresholdStr)) {
            throw new StatusException("ExamRecordDataServiceImpl-calculateFaceCheckResult-001", ExamProperties.WARN_THRESHOLD.getDesc() + "未设置");
        }
        double warnThreshold = Double.parseDouble(warnThresholdStr);
        //人脸真实性(百度活体检测)通过阀值
        String liveWarnThresholdStr = CacheHelper.getExamOrgProperty(examId, examRecordData.getOrgId(),
                ExamProperties.LIVING_WARN_THRESHOLD.name()).getValue();
        if (CommonUtil.isBlank(liveWarnThresholdStr)) {
            throw new StatusException("ExamRecordDataServiceImpl-calculateFaceCheckResult-002", ExamProperties.LIVING_WARN_THRESHOLD.getDesc() + "未设置");
        }
        double livenessThreshold = Double.parseDouble(liveWarnThresholdStr);

        if (warnThreshold == 0d && livenessThreshold > 0d) {
            examRecordData.setIsWarn(examRecordData.getBaiduFaceLivenessSuccessPercent() < livenessThreshold);
        } else if (warnThreshold > 0d && livenessThreshold == 0d) {
            examRecordData.setIsWarn(examRecordData.getFaceSuccessPercent() < warnThreshold);
        } else if (warnThreshold > 0d && livenessThreshold > 0d) {
            boolean isWarn = examRecordData.getFaceSuccessPercent() < warnThreshold ||
                    examRecordData.getBaiduFaceLivenessSuccessPercent() < livenessThreshold;
            examRecordData.setIsWarn(isWarn);
        }
        calculateFaceCheckResultInfo.setExamRecordData(examRecordData);
        return calculateFaceCheckResultInfo;
    }

    /**
     * 计算人脸检测数据
     * 陌生人记录数、成功次数、失败次数、成功率
     *
     * @param examRecordData
     * @param examCaptureEntityList
     * @return
     */
    private ExamRecordDataEntity computeFaceCheckData(ExamRecordDataEntity examRecordData, List<ExamCaptureEntity> examCaptureEntityList) {
        int strangerCount = 0;    // 人脸比较陌生人记录数
        int succCount = 0;        // 人脸比较成功次数
        int falseCount = 0;        // 人脸比较失败次数
        double succPercent = 0d;    // 人脸比较成功率
        int livenessSuccessCount = 0;//百度活体检测成功次数
        double livenessSuccessPercent = 0D;//百度活体检测成功率
        for (ExamCaptureEntity examCaptureEntity : examCaptureEntityList) {
            if (examCaptureEntity.getIsPass() != null && examCaptureEntity.getIsPass()) {
                succCount++;
            } else {
                falseCount++;
            }
            if (examCaptureEntity.getIsStranger() != null && examCaptureEntity.getIsStranger()) {
                strangerCount++;
            }
            livenessSuccessCount += isFacelivenessSuccess(examCaptureEntity);
        }
        int allNum = examCaptureEntityList.size();
        BigDecimal bg = new BigDecimal(((double) succCount / allNum) * 100);
        succPercent = bg.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();// 人脸比较成功率
        examRecordData.setFaceTotalCount(allNum);//检测总次数
        examRecordData.setFaceSuccessPercent(succPercent);//成功率
        examRecordData.setFaceStrangerCount(strangerCount);//有陌生人的次数
        examRecordData.setFaceSuccessCount(succCount);//成功次数
        examRecordData.setFaceFailedCount(falseCount);//失败次数
        //人脸关键点坐标相似比率-废弃
        //examRecordDataEntity.setFaceLandmarkVal(analyseLankmark(examCaptureEntityList));
        BigDecimal livenessSuccessBg = new BigDecimal(((double) livenessSuccessCount / allNum) * 100);
        livenessSuccessPercent = livenessSuccessBg.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();//百度活体检测成功率
        examRecordData.setBaiduFaceLivenessSuccessPercent(livenessSuccessPercent);
        return examRecordData;
    }

    private int isFacelivenessSuccess(ExamCaptureEntity examCapture) {
        if (StringUtils.isNotBlank(examCapture.getFacelivenessResult())) {
            String livenessJson = examCapture.getFacelivenessResult();
            JSONObject jsonObject;
            try {
                jsonObject = new JSONObject(livenessJson);
                if (jsonObject.has("error_code") && jsonObject.getInt("error_code") == 0 && jsonObject.has("result")) {
                    JSONObject resultJson = jsonObject.getJSONObject("result");
                    if (resultJson.has("face_liveness")) {
                        double faceLivenessVal = resultJson.getDouble("face_liveness");
                        double baiduFacelivenessThreshold = Double.parseDouble(PropertyHolder.getString("$baidu.faceliveness.threshold"));
                        if (faceLivenessVal > baiduFacelivenessThreshold) {
                            return 1;
                        }
                    }
                }
            } catch (JSONException e) {
                e.printStackTrace();
                return 0;
            }

        }
        return 0;
    }

    @Override
    public List<ExamRecordDataEntity> findByStatusAndExamTypeIn(ExamRecordStatus examRecordStatus, List<String> examTypeList) {
        Searcher searchers = new Searcher().in("examRecord.examType", examTypeList)
                .eq("examRecordStatus", examRecordStatus.name());
        Specification<ExamRecordDataEntity> spec = SpecUtils.buildSearchers(ExamRecordDataEntity.class, searchers.build());
        return examRecordDataRepo.findAll(spec);
    }

    @Override
    public List<ExamRecordDataEntity> findValidExamRecordDataByExamStudentId(Long examStudentId) {
        Searcher searchers = new Searcher().eq("examRecord.examStudentId", examStudentId);
        Specification<ExamRecordDataEntity> spec = SpecUtils.buildSearchers(ExamRecordDataEntity.class, searchers.build());
        List<ExamRecordDataEntity> examRecordDataList = examRecordDataRepo.findAll(spec);
        //第一次过滤:正常结束或者被系统处理的
        List<ExamRecordDataEntity> firstFilteredexamRecordDataList = examRecordDataList.stream().filter(examRecordData -> {
            return examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_END
                    || examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_OVERDUE;
        }).collect(Collectors.toList());
        /**
         * 第二次过滤:
         * 1.没有违纪的
         * 2.无异常或有异常已审核通过
         */
        if (firstFilteredexamRecordDataList != null && firstFilteredexamRecordDataList.size() > 0) {
            return firstFilteredexamRecordDataList.stream().filter(examRecordData -> {
                return !examRecordData.getIsIllegality() &&
                        (!examRecordData.getIsWarn() || (examRecordData.getIsWarn() && examRecordData.getIsAudit()));
            }).collect(Collectors.toList());
        }

        return null;
    }

    @Override
    public ExamRecordDataEntity createOfflineExamRecordData(ExamStudentEntity examStudent,
                                                            ExamBean examBean,
                                                            CourseBean courseBean,
                                                            String basePaperId,
                                                            String paperStructId, Boolean fullyObjective) {
        ExamRecordDataEntity examRecordDataEntity = new ExamRecordDataEntity();
        examRecordDataEntity.setExamId(examBean.getId());
        examRecordDataEntity.setExamType(ExamType.strToEnum(examBean.getExamType()));

        examRecordDataEntity.setExamStudentId(examStudent.getExamStudentId());
        examRecordDataEntity.setStudentId(examStudent.getStudentId());
        examRecordDataEntity.setStudentCode(examStudent.getStudentCode());
        examRecordDataEntity.setStudentName(examStudent.getStudentName());
        examRecordDataEntity.setIdentityNumber(examStudent.getIdentityNumber());
        examRecordDataEntity.setOrgId(examStudent.getOrgId());
        examRecordDataEntity.setRootOrgId(examStudent.getRootOrgId());

        examRecordDataEntity.setCourseId(courseBean.getId());
        examRecordDataEntity.setCourseLevel(examStudent.getCourseLevel());
        examRecordDataEntity.setBasePaperId(basePaperId);

        examRecordDataEntity.setPaperType(examStudent.getPaperType());
        examRecordDataEntity.setPaperStructId(paperStructId);

        examRecordDataEntity.setInfoCollector(examStudent.getInfoCollector());
//        examRecordDataEntity.setExamRecordDataId(examRecord.getId());
        examRecordDataEntity.setStartTime(new Date());
        examRecordDataEntity.setExamRecordStatus(ExamRecordStatus.EXAM_ING);
        examRecordDataEntity.setExamOrder(1);
        examRecordDataEntity.setIsAllObjectivePaper(fullyObjective);
        return examRecordDataRepo.save(examRecordDataEntity);
    }

    @Override
    public ExamRecordDataEntity examRecordAutoAudit(Boolean isNoPhotoAndIllegality, ExamRecordDataEntity examRecordData) {
        //无照片违纪自动审核
        if (isNoPhotoAndIllegality != null && isNoPhotoAndIllegality) {
            examAuditService.saveExamAuditByNoPhoto(examRecordData.getId());
            examRecordData.setIsAudit(true);
        } else {
            //活体检测失败违纪自动审核
            if (examRecordData.getFaceVerifyResult() != null
                    && examRecordData.getFaceVerifyResult() == IsSuccess.FAILED
                    && examRecordData.getIsIllegality()) {
                examAuditService.saveExamAuditByFaceVerifyFailed(examRecordData.getId());
                examRecordData.setIsAudit(true);
            }
        }
        return examRecordData;
    }

    @Override
    public boolean isExamRecordEnded(ExamRecordDataEntity examRecordData) {
        //如果考试记录状态为已处理，则直接返回true.
        if (examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_END ||
                examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_OVERDUE ||
                examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_INVALID) {
            return true;
        }
        return false;
    }

    @Override
    public boolean isExamRecordEnded(Long examRecordDataId) {
        log.debug("进入isExamRecordEnded方法，examRecordDataId=" + examRecordDataId);
        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);
        if (examRecordData == null) {
            throw new StatusException("", "找不到id为:" + examRecordDataId + "的考试记录");
        }
        return isExamRecordEnded(examRecordData);
    }
}
