package cn.com.qmth.examcloud.core.oe.student.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.core.oe.common.base.utils.QuestionTypeUtil;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamQuestionEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordQuestionsEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamType;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordQuestionsRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamScoreRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamStudentRepo;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamSessionInfo;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamStudentQuestionInfo;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamSessionInfoService;
import cn.com.qmth.examcloud.core.questions.api.ExtractConfigCloudService;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultPaper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionGroup;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionStructureWrapper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionUnitWrapper;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestionStructure;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestionUnit;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.QuestionCacheBean;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import com.alibaba.fastjson.JSON;
import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author chenken
 * @date 2018年9月5日 下午3:23:26
 * @company QMTH
 * @description 考试作答归档服务实现
 */
@Service("examRecordQuestionsService")
public class ExamRecordQuestionsServiceImpl implements ExamRecordQuestionsService {

    @Autowired
    private ExamRecordQuestionsRepo examRecordQuestionsRepo;

    @Autowired
    private ExamScoreRepo examScoreRepo;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamSessionInfoService examSessionInfoService;

    @Autowired
    private ExtractConfigCloudService extractConfigCloudService;

    @Autowired
    private ExamStudentRepo examStudentRepo;

    //本地题干缓存，最大200道题，默认缓存120秒
    private Cache<String, String> questionContentCache = CacheBuilder.newBuilder().maximumSize(200)
            .expireAfterWrite(120, TimeUnit.SECONDS).build();

//    @Value("${exam_question_key_prefix}")
//    private String examQuestionKeyPrefix;

    @Override
    public Integer calculationSubjectiveAnswerLength(Long examRecordDataId) {
        ExamRecordQuestionsEntity examRecordQuesitonsEntity = getExamRecordQuestionsAndFixExamRecordDataIfNecessary(examRecordDataId);
        if (examRecordQuesitonsEntity == null || examRecordQuesitonsEntity.getExamQuestionEntities() == null ||
                examRecordQuesitonsEntity.getExamQuestionEntities().isEmpty()) {
            return 0;
        }
        List<ExamQuestionEntity> examQuestionList = examRecordQuesitonsEntity.getExamQuestionEntities();

		/*ExamRecordQuestionsEntity examRecordQuestions = examRecordQuestionsRepo.findByExamRecordDataId(examRecordDataId);
		List<ExamQuestionEntity> examQuestionEntities = examRecordQuestions.getExamQuestionEntities();*/
        int answerLength = 0;
        for (ExamQuestionEntity examQuestionEntity : examQuestionList) {
            if (!QuestionTypeUtil.isObjectiveQuestion(examQuestionEntity.getQuestionType())
                    && examQuestionEntity.getStudentAnswer() != null) {
                answerLength += examQuestionEntity.getStudentAnswer().length();
            }
        }
        return answerLength;
    }

    @Override
    public ExamRecordQuestionsEntity createExamRecordQuestions(Long examRecordDataId, DefaultPaper defaultPaper) {
        ExamRecordQuestionsEntity examRecordQuestionsEntity = new ExamRecordQuestionsEntity();

        List<ExamQuestionEntity> examQuestionEntityList = new ArrayList<ExamQuestionEntity>();
        List<DefaultQuestionGroup> defaultQuestionGroups = defaultPaper.getQuestionGroupList();
        int order = 0;
        for (int i = 0; i < defaultQuestionGroups.size(); i++) {
            DefaultQuestionGroup defaultQuestionGroup = defaultQuestionGroups.get(i);
            List<DefaultQuestionStructureWrapper> defaultQuestionStructureWrappers = defaultQuestionGroup.getQuestionWrapperList();
            for (DefaultQuestionStructureWrapper defaultQuestionStructureWrapper : defaultQuestionStructureWrappers) {
                List<DefaultQuestionUnitWrapper> questionUnitWrapperList = defaultQuestionStructureWrapper.getQuestionUnitWrapperList();
                for (DefaultQuestionUnitWrapper defaultQuestionUnitWrapper : questionUnitWrapperList) {
                    ExamQuestionEntity examQuestionEntity = new ExamQuestionEntity();
                    examQuestionEntity.setExamRecordDataId(examRecordDataId);
                    examQuestionEntity.setMainNumber(i + 1);
                    examQuestionEntity.setOrder(++order);
                    examQuestionEntity.setQuestionId(defaultQuestionStructureWrapper.getQuestionId());
                    examQuestionEntity.setQuestionScore(defaultQuestionUnitWrapper.getQuestionScore());
                    examQuestionEntity.setQuestionType(defaultQuestionUnitWrapper.getQuestionType());
                    examQuestionEntity.setOptionPermutation(defaultQuestionUnitWrapper.getOptionPermutation());
                    examQuestionEntity.setAudioPlayTimes(null);
                    examQuestionEntity.setAnswerType(defaultQuestionUnitWrapper.getAnswerType());

                    examQuestionEntityList.add(examQuestionEntity);
                }
            }
        }

        examRecordQuestionsEntity.setExamQuestionEntities(examQuestionEntityList);
        examRecordQuestionsEntity.setExamRecordDataId(examRecordDataId);
        examRecordQuestionsEntity.setCreationTime(new Date());
        ExamRecordQuestionsEntity saveResult = examRecordQuestionsRepo.save(examRecordQuestionsEntity);
//        redisTemplate.opsForList().leftPushAll(examQuestionKeyPrefix+examRecordDataId,examQuestionEntityList);
        return saveResult;
    }

    /**
     * 获取考试作答记录并修复考试记录数据(如有必要)
     *
     * @param examRecordData 考试记录
     * @return
     */
    @Override
    public ExamRecordQuestionsEntity getExamRecordQuestionsAndFixExamRecordDataIfNecessary(ExamRecordDataEntity examRecordData) {
        if (examRecordData == null) {
            throw new StatusException("201101", "examRecordData参数不允许为空");
        }
        Long examRecordDataId = examRecordData.getId();
        String examRecordQuestionId = examRecordData.getExamRecordQuestionsId();

        //如果考试作答记录id不为空，则根据id查询考试作答记录
        if (StringUtils.isNotEmpty(examRecordQuestionId)) {
            return GlobalHelper.getEntity(examRecordQuestionsRepo, examRecordQuestionId,
                    ExamRecordQuestionsEntity.class);
        }
        //如果考试作答记录id为空，则根据考试记录id获取考试作答记录，并将考试作答记录id保存至examRecordData表中
        else {
            ExamRecordQuestionsEntity examRecordQuestionsEntity = examRecordQuestionsRepo.findByExamRecordDataId(examRecordDataId);

            //将考试作答记录id保存至examRecordData表中,目的：纠正历史数据
            examRecordDataRepo.updateExamRecordDataQuestionIdById(examRecordQuestionsEntity.getId(), examRecordData.getId());
            return examRecordQuestionsEntity;
        }
    }

    @Override
    public ExamRecordQuestionsEntity getExamRecordQuestionsAndFixExamRecordDataIfNecessary(Long examRecordDataId) {
        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo,
                examRecordDataId, ExamRecordDataEntity.class);
        return getExamRecordQuestionsAndFixExamRecordDataIfNecessary(examRecordData);
    }

    @Override
    public void submitQuestionAnswer(Long studentId, List<ExamStudentQuestionInfo> examQuestionInfos) {
        ExamSessionInfo examSessionInfo = examSessionInfoService.getExamSessionInfo(studentId);
        if (null == examSessionInfo) {
            throw new StatusException("200001", "考试会话已过期");
        }
        long examRecordDataId = examSessionInfo.getExamRecordDataId();

        ExamRecordQuestionsEntity examRecordQuesitonsEntity = getExamRecordQuestionsAndFixExamRecordDataIfNecessary(examRecordDataId);
        List<ExamQuestionEntity> examQuestionEntities = examRecordQuesitonsEntity.getExamQuestionEntities();

        for (ExamStudentQuestionInfo examQuestionInfo : examQuestionInfos) {
            for (ExamQuestionEntity examQuestion : examQuestionEntities) {
                if (examQuestion.getOrder().equals(examQuestionInfo.getOrder())) {
                    examQuestion.setStudentAnswer(examQuestionInfo.getStudentAnswer());
                    examQuestion.setIsSign(examQuestionInfo.getIsSign());
                    examQuestion.setIsAnswer(StringUtils.isNotBlank(examQuestion.getStudentAnswer()));
                    examQuestion.setAudioPlayTimes(examQuestionInfo.getAudioPlayTimes());
                    break;
                }
            }
        }
        examRecordQuestionsRepo.save(examRecordQuesitonsEntity);
    }

    @Override
    public String getQuestionContent(Long studentId, String questionId) {
        ExamSessionInfo examSessionInfo = examSessionInfoService.getExamSessionInfo(studentId);
        if (examSessionInfo == null) {
            throw new StatusException("100002", "考试已结束，不允许获取试题内容");
        }

        //本地缓存key规则：examId_courseCode_paperType_questionId
        String cacheKey = examSessionInfo.getExamId() + "_" + examSessionInfo.getCourseCode()
                + "_" + examSessionInfo.getPaperType() + "_" + questionId;
        String contentJson = questionContentCache.getIfPresent(cacheKey);

        //如果本地缓存中存在，则从本地缓存中获取
        if (StringUtils.isNotEmpty(contentJson)){
            return contentJson;
        }
        //如果本地缓存不存在，则从redis中获取, 并在本地缓存存在存储一份
        else {
            QuestionCacheBean getQuestionResp = CacheHelper.getQuestion(examSessionInfo.getExamId(),
                    examSessionInfo.getCourseCode(), examSessionInfo.getPaperType(), questionId);
            DefaultQuestionStructure questionStructure = getQuestionResp.getDefaultQuestion().getMasterVersion();
            List<DefaultQuestionUnit> questionUnits = questionStructure.getQuestionUnitList();

            //在线考试,清除答案
            if (examSessionInfo.getExamType().equals(ExamType.ONLINE.name())) {
                for (DefaultQuestionUnit questionUnit : questionUnits) {
                    questionUnit.setRightAnswer(null);
                }
            }
            String resultJson = JsonUtil.toJson(questionStructure);
            questionContentCache.put(cacheKey, resultJson);
            return resultJson;
        }
    }
}
