package cn.com.qmth.examcloud.core.oe.student.service.impl;

import cn.com.qmth.examcloud.core.oe.common.base.utils.QuestionTypeUtil;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamQuestionEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordQuestionsEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamScoreEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordQuestionsRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamScoreRepo;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamSessionInfo;
import cn.com.qmth.examcloud.core.oe.student.bean.ObjectiveScoreInfo;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamScoreService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamSessionInfoService;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.CourseCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.QuestionAnswerCacheBean;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;


/**
 * @author chenken
 * @date 2018年9月3日 上午10:02:38
 * @company QMTH
 * @description 考试记录得分(总分)服务实现
 */
@Service("examScoreService")
public class ExamScoreServiceImpl implements ExamScoreService {

    @Autowired
    private ExamScoreRepo examScoreRepo;

    @Autowired
    private ExamRecordQuestionsService examRecordQuestionsService;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamRecordDataService examRecordDataService;

    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    RedisClient redisClient;
    @Autowired
    private ExamSessionInfoService examSessionInfoService;
    @Autowired
    private ExamRecordQuestionsRepo examRecordQuestionsRepo;

    private static final Logger log = LoggerFactory.getLogger(ExamScoreServiceImpl.class);

    /**
     * 保存考试总成绩
     *
     * @param examRecordData
     * @return
     */
    @Override
    public ExamScoreEntity saveExamScore(ExamRecordDataEntity examRecordData) {
        long st = System.currentTimeMillis();
        long startTime = System.currentTimeMillis();
        //获取考试作答记录并修复考试记录数据(如有必要)
        ExamRecordQuestionsEntity examRecordQuestionsEntity = examRecordQuestionsService.
                getExamRecordQuestionsAndFixExamRecordDataIfNecessary(examRecordData);
        if (log.isDebugEnabled()) {
            log.debug("1 [SAVE_EXAM_SCORE]获取考试作答记录并修复考试记录数据耗时:" + (System.currentTimeMillis() - startTime) + " ms");
        }

        // 考生作答记录明细
        List<ExamQuestionEntity> examQuestionList = examRecordQuestionsEntity.getExamQuestionEntities();

        double studentObjectiveScoreTotal = 0d;//客观题总分
        int succQuestionNum = 0;//答题正确数
        int objectiveQuestionsNum = 0;//客观题总数
        //是否需要更新客观题答案
        Boolean needUpdateObjectiveAnswer=false;
        //获取考试会话
        ExamSessionInfo examSessionInfo = examSessionInfoService.getExamSessionInfo(examRecordData.getStudentId());
        for (int i = 0; i < examQuestionList.size(); i++) {
            ExamQuestionEntity examQuestionEntity = examQuestionList.get(i);
            String courseCode;
            //自动服务交卷时，会话可能已经过期,需要重新查询课程
            if (examSessionInfo == null) {
                CourseCacheBean courseCache = CacheHelper.getCourse(examRecordData.getCourseId());
                courseCode = courseCache.getCode();
            }
            //会话未过期，可以直接从会话中获取
            else {
                courseCode = examSessionInfo.getCourseCode();
            }

            //只计算客观点题分数
            if (QuestionTypeUtil.isObjectiveQuestion(examQuestionEntity.getQuestionType())) {
                objectiveQuestionsNum++;

                //如果标准答案为空，则更新标准答案
                if (StringUtils.isEmpty(examQuestionEntity.getCorrectAnswer())) {
                    needUpdateObjectiveAnswer=true;
                    updateCorrectAnswer(examRecordData.getExamId(),courseCode,examRecordData.getPaperType(),
                            examQuestionEntity.getQuestionId(),examRecordQuestionsEntity);
                }

                if (examQuestionEntity.getStudentAnswer() != null
                        && examQuestionEntity.getCorrectAnswer() != null
                        && examQuestionEntity.getStudentAnswer().equals(examQuestionEntity.getCorrectAnswer())) {
                    //小题分数
                    double questionScore = examQuestionEntity.getQuestionScore().doubleValue();
                    BigDecimal bigDecimalQuestionScore = new BigDecimal(Double.toString(questionScore));
                    BigDecimal bigDecimalObjectiveScoreTotal = new BigDecimal(Double.toString(studentObjectiveScoreTotal));
                    studentObjectiveScoreTotal = bigDecimalQuestionScore.add(bigDecimalObjectiveScoreTotal).doubleValue();
                    succQuestionNum++;
                }
            }
        }
        if (needUpdateObjectiveAnswer){
            examRecordQuestionsRepo.save(examRecordQuestionsEntity);
        }

        double objectiveAccuracy = 0;//客观题答题正确率
        if (succQuestionNum > 0 && objectiveQuestionsNum > 0) {
            objectiveAccuracy = Double.valueOf(new DecimalFormat("#.00").format(succQuestionNum * 100D / objectiveQuestionsNum));
        }

        Long examRecordDataId = examRecordData.getId();

        startTime = System.currentTimeMillis();
        //如果考试成绩不存在，则新增，否则更新数据
        ExamScoreEntity examScoreEntity = examScoreRepo.findByExamRecordDataId(examRecordDataId);
        if (log.isDebugEnabled()) {
            log.debug("2 [SAVE_EXAM_SCORE]根据考试记录id获取考试分数数据耗时:" + (System.currentTimeMillis() - startTime) + " ms");
        }

        if (examScoreEntity == null) {
            examScoreEntity = new ExamScoreEntity();
            examScoreEntity.setExamRecordDataId(examRecordDataId);
        }
        examScoreEntity.setObjectiveScore(studentObjectiveScoreTotal);
        examScoreEntity.setObjectiveAccuracy(objectiveAccuracy);
        examScoreEntity.setTotalScore(studentObjectiveScoreTotal);//交卷时,总分=客观分得分

        startTime = System.currentTimeMillis();
        //保存考试成绩
        ExamScoreEntity resultEntity = examScoreRepo.save(examScoreEntity);
        if (log.isDebugEnabled()) {
            log.debug("3 [SAVE_EXAM_SCORE]保存考试成绩耗时:" + (System.currentTimeMillis() - startTime) + " ms");
        }

        log.debug("4 [SAVE_EXAM_SCORE]合计 耗时:" + (System.currentTimeMillis() - st) + " ms");

        return resultEntity;
    }

    /**
     * 更新客观题答案
     * @param examId
     * @param courseCode
     * @param paperType
     * @param questionId
     * @param examRecordQuestionsEntity
     * @return
     */
    private void updateCorrectAnswer(Long examId, String courseCode, String paperType,
                                    String questionId,ExamRecordQuestionsEntity examRecordQuestionsEntity) {
        //更新客观题答案
        QuestionAnswerCacheBean questionAnswerCache = CacheHelper.getQuestionAnswer(questionId);
        List<String> rightAnswerList = questionAnswerCache.getRightAnswers();
        //最小维度的小题单元集合
        List<ExamQuestionEntity> questionUnitList = examRecordQuestionsEntity.getExamQuestionEntities().stream().
                filter(p -> p.getQuestionId().equals(questionId)).
                sorted((o1, o2) -> o1.getOrder().intValue() - o2.getOrder().intValue()).collect(Collectors.toList());

        for (int i = 0; i < questionUnitList.size(); i++) {
            ExamQuestionEntity examQuestionEntity = questionUnitList.get(i);
            examQuestionEntity.setCorrectAnswer(rightAnswerList.get(i));
        }
    }

    @Override
    public void createExamScoreWithOffline(Long examRecordDataId) {
        ExamScoreEntity examScoreEntity = new ExamScoreEntity();
        examScoreEntity.setExamRecordDataId(examRecordDataId);
        examScoreEntity.setObjectiveScore(0D);
        examScoreEntity.setSubjectiveScore(0D);
        examScoreEntity.setTotalScore(0D);
        examScoreRepo.save(examScoreEntity);
    }

    @Override
    public List<ExamScoreEntity> queryExamScoreListByExamStudentId(Long examStudentId) {
        List<ExamRecordDataEntity> examRecordDataEntities = examRecordDataService.findValidExamRecordDataByExamStudentId(examStudentId);
        if (examRecordDataEntities == null || examRecordDataEntities.size() == 0) {
            return null;
        }
        List<Long> examRecordDataIds = examRecordDataEntities.stream().map(ExamRecordDataEntity::getId).collect(Collectors.toList());
        return examScoreRepo.findByExamRecordDataIdIn(examRecordDataIds);
    }

    @Override
    public List<ObjectiveScoreInfo> queryObjectiveScoreList(Long examStudentId) {
        List<ExamRecordDataEntity> examRecordDataList = examRecordDataRepo.findByExamStudentId(examStudentId);
        //过滤已完成的考试记录(包括违纪的)
        examRecordDataList = examRecordDataList.stream().filter((o -> {
            return o.getExamRecordStatus() == ExamRecordStatus.EXAM_END ||
                    o.getExamRecordStatus() == ExamRecordStatus.EXAM_OVERDUE ||
                    o.getExamRecordStatus() == ExamRecordStatus.EXAM_HAND_IN ||
                    o.getExamRecordStatus() == ExamRecordStatus.EXAM_AUTO_HAND_IN;
        })).collect(Collectors.toList());

        List<ObjectiveScoreInfo> objectiveScoreInfoList = new ArrayList<ObjectiveScoreInfo>();
        for (ExamRecordDataEntity examRecordDataEntity : examRecordDataList) {
            ObjectiveScoreInfo objectiveScoreInfo = new ObjectiveScoreInfo();
            objectiveScoreInfo.setExamRecordDataId(examRecordDataEntity.getId());
            objectiveScoreInfo.setExamOrder(examRecordDataEntity.getExamOrder());
            objectiveScoreInfo.setStartTime(examRecordDataEntity.getStartTime());
            objectiveScoreInfo.setEndTime(examRecordDataEntity.getEndTime());

            //如果考试没有结束，则只能返回部分数据
            if (!examRecordDataService.isExamRecordEnded(examRecordDataEntity)) {
                objectiveScoreInfo.setIsExamEnded(false);
                objectiveScoreInfoList.add(objectiveScoreInfo);
                continue;
            } else {
                objectiveScoreInfo.setIsExamEnded(true);
            }

            if (!examRecordDataEntity.getIsIllegality()) {
                if (examRecordDataEntity.getIsWarn() && !examRecordDataEntity.getIsAudit()) {
                    objectiveScoreInfo.setIsAuditing(true);
                } else if (!examRecordDataEntity.getIsWarn() || (examRecordDataEntity.getIsWarn() && examRecordDataEntity.getIsAudit())) {
                    ExamScoreEntity examScore = examScoreRepo.findByExamRecordDataId(examRecordDataEntity.getId());
                    objectiveScoreInfo.setIsAuditing(false);
                    objectiveScoreInfo.setObjectiveScore(examScore.getObjectiveScore());
                }
                objectiveScoreInfo.setIsIllegality(false);
            } else {
                objectiveScoreInfo.setIsIllegality(true);
            }
            objectiveScoreInfoList.add(objectiveScoreInfo);
        }
        return objectiveScoreInfoList;
    }

}
