package cn.com.qmth.examcloud.core.oe.student.service.impl;

import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.oe.common.base.utils.CommonUtil;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.CourseLevel;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamProperties;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamType;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamStudentRepo;
import cn.com.qmth.examcloud.core.oe.common.service.GainBaseDataService;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamStudentInfo;
import cn.com.qmth.examcloud.core.oe.student.service.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.core.oe.student.service.ExamStudentService;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamBean;
import cn.com.qmth.examcloud.examwork.api.bean.ExamSpecialSettingsBean;
import cn.com.qmth.examcloud.examwork.api.request.GetOngoingExamListReq;
import cn.com.qmth.examcloud.examwork.api.response.GetOngoingExamListResp;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.OrgCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.StudentCacheBean;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author chenken
 * @date 2018年8月13日 下午2:46:10
 * @company QMTH
 * @description 考生服务实现
 */
@Service("examStudentService")
public class ExamStudentServiceImpl implements ExamStudentService {

    @Autowired
    private ExamStudentRepo examStudentRepo;

    @Autowired
    private ExamCloudService examCloudService;

    @Autowired
    private GainBaseDataService gainBaseDataService;

    /**
     * 根据学生id获取考试列表
     *
     * @param studentId 学生id
     */
    @Override
    public List<ExamStudentInfo> queryOnlineExamList(Long studentId) {
        StudentCacheBean studentBean = CacheHelper.getStudent(studentId);

        //获取可以考的和即将考的考试Id
        GetOngoingExamListReq getOngoingExamListReq = new GetOngoingExamListReq();
        getOngoingExamListReq.setExamType(ExamType.ONLINE.name());
        getOngoingExamListReq.setRootOrgId(studentBean.getRootOrgId());
        getOngoingExamListReq.setOrgId(studentBean.getOrgId());
        GetOngoingExamListResp getOngoingExamListResp = examCloudService.getOngoingExamList(getOngoingExamListReq);

        //获取学生所在组织机构的所有考试列表集合（虽然名字起的特殊考试设置，事实上取的就是实际的可考的考试列表，可能是考试中心特殊设置的考试时间，也可能不是）
        List<ExamSpecialSettingsBean> examSpecialSettingsBeanList = getOngoingExamListResp.getExamSpecialSettingsList();
        if (examSpecialSettingsBeanList == null || examSpecialSettingsBeanList.size() == 0) {
            return null;
        }
        List<Long> examIds = examSpecialSettingsBeanList.stream().map(ExamSpecialSettingsBean::getExamId).collect(Collectors.toList());

        //根据studentId和考试Id查询考生()
//		List<ExamStudentEntity> examStudents = examStudentRepo.findByStudentIdAndExamIdIn(studentId, examIds);
        //只查没有禁用的考生
        List<ExamStudentEntity> examStudents = examStudentRepo.findByStudentIdAndEnableAndExamIdIn(studentId, true, examIds);
        List<ExamStudentInfo> examStudentDtoList = new ArrayList<ExamStudentInfo>();
        for (ExamStudentEntity examStudent : examStudents) {
            Stream<ExamSpecialSettingsBean> examSpecialSettingsBeanStream = examSpecialSettingsBeanList.stream().filter(examSpecialSettingsBean -> {
                return examSpecialSettingsBean.getExamId().longValue() == examStudent.getExamId().longValue();
            });
            if (examSpecialSettingsBeanStream != null) {
                ExamSpecialSettingsBean examSpecialSettingsBean = examSpecialSettingsBeanStream.findFirst().get();
                examStudentDtoList.add(assemblingExamStudentDto(examStudent, examSpecialSettingsBean));
            }
        }
        return examStudentDtoList;
    }

    private ExamStudentInfo assemblingExamStudentDto(ExamStudentEntity examStudent, ExamSpecialSettingsBean examSpecialSettingsBean) {
        ExamStudentInfo examStudentInfo = new ExamStudentInfo();
        examStudentInfo.setExamStudentId(examStudent.getExamStudentId());
        examStudentInfo.setStudentCode(examStudent.getStudentCode());
        examStudentInfo.setStudentName(examStudent.getStudentName());
        examStudentInfo.setRootOrgId(examStudent.getRootOrgId());
        examStudentInfo.setIdentityNumber(examStudent.getIdentityNumber());

        CourseBean courseBean = ExamCacheTransferHelper.getCachedCourse(examStudent.getCourseId());
        examStudentInfo.setCourseName(courseBean.getName());
        examStudentInfo.setCourseCode(courseBean.getCode());
        examStudentInfo.setCourseLevel(CourseLevel.getCourseLevel(courseBean.getLevel()).getTitle());
        examStudentInfo.setCourseId(examStudent.getCourseId());
        examStudentInfo.setSpecialtyName(examStudent.getSpecialtyName());
        examStudentInfo.setOrgId(examStudent.getOrgId());

        OrgCacheBean orgBean = gainBaseDataService.getOrgBean(examStudent.getOrgId());
        ExamBean examBean = ExamCacheTransferHelper.getCachedExam(examStudent.getExamId());

        examStudentInfo.setOrgName(orgBean.getName());
        examStudentInfo.setExamId(examBean.getId());
        examStudentInfo.setExamName(examBean.getName());
        examStudentInfo.setStartTime(examSpecialSettingsBean.getBeginTime());//考试开始时间设置
        examStudentInfo.setEndTime(examSpecialSettingsBean.getEndTime());//考试结束时间设置
        examStudentInfo.setAllowExamCount(countExamTimes(examStudent, examBean));
        examStudentInfo.setPaperMins(examBean.getDuration());

        //是否启用人脸识别
        String isFaceEnable = CacheHelper.getExamOrgProperty(examBean.getId(), examStudent.getOrgId(),
                ExamProperties.IS_FACE_ENABLE.name()).getValue();
        if (StringUtils.isBlank(isFaceEnable)) {
            examStudentInfo.setFaceEnable(false);
        } else {
            examStudentInfo.setFaceEnable(Boolean.valueOf(isFaceEnable));
        }

        //进入考试是否验证人脸识别(强制、非强制)
        String isFaceCheck = CacheHelper.getExamOrgProperty(examBean.getId(), examStudent.getOrgId(),
                ExamProperties.IS_FACE_CHECK.name()).getValue();
        if (StringUtils.isBlank(isFaceCheck)) {
            examStudentInfo.setFaceCheck(false);
        } else {
            examStudentInfo.setFaceCheck(Boolean.valueOf(isFaceCheck));
        }
        //是否显示客观分
        String isObjScoreView = CacheHelper.getExamOrgProperty(examBean.getId(), examStudent.getOrgId(),
                ExamProperties.IS_OBJ_SCORE_VIEW.name()).getValue();
        if (StringUtils.isBlank(isObjScoreView)) {
            examStudentInfo.setIsObjScoreView(false);
        } else {
            examStudentInfo.setIsObjScoreView(Boolean.valueOf(isObjScoreView));
        }
        return examStudentInfo;
    }

    @Override
    public Integer countExamTimes(ExamStudentEntity examStudentInfo, ExamBean examBean) {
        if (ExamType.OFFLINE.name().equals(examBean.getExamType())) {
            return 1;
        }
        //考试批次中设置的考试次数
        int canExamTimes = examBean.getExamTimes().intValue();
        //正常考试的次数
        int normalExamTimes = examStudentInfo.getNormalExamTimes();
        /**
         * 如果设置了重考，重考未完成;
         * 且正常考试的次数大于等于设定的次数
         * 将考试剩余次数设置为1
         */
        if (reexamineNotCompleted(examStudentInfo) && normalExamTimes >= canExamTimes) {
            return 1;
        }
        int allowExamCount = canExamTimes - normalExamTimes;
        return allowExamCount < 0 ? 0 : allowExamCount;
    }

    /**
     * 设置了重考，且重考未完成
     *
     * @param examStudentInfo
     * @return
     */
    private boolean reexamineNotCompleted(ExamStudentEntity examStudentInfo) {
        if (examStudentInfo.getIsReExamine() != null && examStudentInfo.getIsReExamine()
                && (examStudentInfo.getReExamineCompleted() == null || !examStudentInfo.getReExamineCompleted())) {
            return true;
        }
        return false;
    }

    @Override
    public int updateExamStudentByStartExam(Long id,Boolean isReExamine,Boolean reExamineCompleted,
                                                          Integer normalExamTimes,final ExamBean examBean) {
        //考试完的最终状态
        Boolean finalIsReExamine=isReExamine;
        Boolean finalReExamineCompleted=reExamineCompleted;
        //考生的已考次数(不包括重考)
        int finalNormalExamTimes=normalExamTimes == null ? 0 : normalExamTimes;
        //考试机会，即可考次数
        long canExamTimes = examBean.getExamTimes();
        //有可能中途改变考试次数
        if (canExamTimes > finalNormalExamTimes) {
            finalNormalExamTimes = finalNormalExamTimes + 1;
            finalIsReExamine = null;
            finalReExamineCompleted = null;
        }

        // 考生开始重考时，将重考已完成设置为true，并重置是否重考
        if (CommonUtil.isTrue(finalIsReExamine)) {
            finalIsReExamine = false;
            finalReExamineCompleted = true;
            finalNormalExamTimes=finalNormalExamTimes+1;
        }
        Date now = new Date();
        //更新相关属性
        examStudentRepo.updateExamStudentStartExamStatusInfo(id, true,
                finalNormalExamTimes, finalIsReExamine, finalReExamineCompleted, now);
        return finalNormalExamTimes;
    }
}
