package cn.com.qmth.examcloud.core.oe.student.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.HttpClientUtil;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordForMarkingEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.OfflineFileEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamType;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordForMarkingRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamStudentRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.OfflineFileRepo;
import cn.com.qmth.examcloud.core.oe.common.service.GainBaseDataService;
import cn.com.qmth.examcloud.core.oe.student.bean.OfflineExamCourseInfo;
import cn.com.qmth.examcloud.core.oe.student.service.*;
import cn.com.qmth.examcloud.core.questions.api.ExtractConfigCloudService;
import cn.com.qmth.examcloud.core.questions.api.request.GetPaperReq;
import cn.com.qmth.examcloud.core.questions.api.response.GetPaperResp;
import cn.com.qmth.examcloud.examwork.api.bean.ExamBean;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.OrgCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.SysPropertyCacheBean;
import cn.com.qmth.examcloud.web.facepp.FaceppClient;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import main.java.com.UpYun;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.compress.utils.IOUtils;
import org.apache.commons.io.FileUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpStatus;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.CookieSpecs;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.io.*;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author chenken
 * @date 2018年9月5日 下午3:24:44
 * @company QMTH
 * @description 离线考试服务实现
 */
@Service("offlineExamService")
public class OfflineExamServiceImpl implements OfflineExamService {

    @Autowired
    private ExamStudentRepo examStudentRepo;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private GainBaseDataService gainBaseDataService;

    @Autowired
    private ExtractConfigCloudService extractConfigCloudService;

    @Autowired
    private ExamRecordDataService examRecordDataService;

    @Autowired
    private ExamScoreService examScoreService;

    @Autowired
    private ExamRecordForMarkingService examRecordForMarkingService;

    @Autowired
    private ExamRecordForMarkingRepo examRecordForMarkingRepo;

    @Autowired
    private OfflineFileRepo offlineFileRepo;

    @Value("${$upyun.site.1.bucketName}")
    private String bucketName;

    @Value("${$upyun.site.1.userName}")
    private String userName;

    @Value("${$upyun.site.1.password}")
    private String password;

    @Value("${app.upyun.uploadUrl}")
    private String upyunUploadUrl;

    @Value("${$upyun.site.1.domain}")
    private String upyunFileUrl;

    @Override
    public List<OfflineExamCourseInfo> getOfflineCourse(Long studentId) {
        List<ExamStudentEntity> examStudents = examStudentRepo.findByStudentId(studentId);
        List<OfflineExamCourseInfo> offlineExamCourseInfoList = new ArrayList<OfflineExamCourseInfo>();
        for (ExamStudentEntity examStudent : examStudents) {
            ExamBean examBean = ExamCacheTransferHelper.getCachedExam(examStudent.getExamId(),
                    examStudent.getOrgId());
            if (!ExamType.OFFLINE.name().equals(examBean.getExamType())) {
                continue;
            }
            if (!examBean.getEnable() || (examBean.getExamLimit() != null && examBean.getExamLimit())) {
                continue;
            }
            if (new Date().before(examBean.getBeginTime()) || examBean.getEndTime().before(new Date())) {
                continue;
            }
            offlineExamCourseInfoList.add(toOfflineExamCourse(examStudent, examBean));
        }
        return offlineExamCourseInfoList;
    }

    private OfflineExamCourseInfo toOfflineExamCourse(ExamStudentEntity examStudent, ExamBean examBean) {
        OfflineExamCourseInfo offlineExamCourseInfo = new OfflineExamCourseInfo();

        CourseBean courseBean = ExamCacheTransferHelper.getCachedCourse(examStudent.getCourseId());
        offlineExamCourseInfo.setCourseCode(courseBean.getCode());
        offlineExamCourseInfo.setCourseLevel(courseBean.getLevel());
        offlineExamCourseInfo.setCourseName(courseBean.getName());
        offlineExamCourseInfo.setExamId(examBean.getId());
        offlineExamCourseInfo.setExamName(examBean.getName());
        offlineExamCourseInfo.setSpecialtyName(examStudent.getSpecialtyName());
        offlineExamCourseInfo.setExamStudentId(examStudent.getExamStudentId());
        offlineExamCourseInfo.setStudentCode(examStudent.getStudentCode());
        offlineExamCourseInfo.setStudentName(examStudent.getStudentName());
        offlineExamCourseInfo.setStartTime(examBean.getBeginTime());
        offlineExamCourseInfo.setEndTime(examBean.getEndTime());
        Date nowDate = new Date();
        if (nowDate.getTime() > offlineExamCourseInfo.getStartTime().getTime()
                && nowDate.getTime() < offlineExamCourseInfo.getEndTime().getTime()) {
            offlineExamCourseInfo.setIsvalid(true);
        } else {
            offlineExamCourseInfo.setIsvalid(false);
        }

        OrgCacheBean orgBean = gainBaseDataService.getOrgBean(examStudent.getOrgId());
        offlineExamCourseInfo.setOrgName(orgBean.getName());

        List<ExamRecordDataEntity> examRecords = examRecordDataRepo.findByExamStudentId(examStudent.getExamStudentId());
        if (examRecords.size() > 0) {
            ExamRecordDataEntity examRecordDataEntity = examRecords.get(0);
            offlineExamCourseInfo.setExamRecordDataId(examRecordDataEntity.getId());
            offlineExamCourseInfo.setStatus(examRecordDataEntity.getExamRecordStatus());
            offlineExamCourseInfo.setPaperId(examRecordDataEntity.getBasePaperId());
            if (examRecordDataEntity.getExamRecordStatus() == ExamRecordStatus.EXAM_END) {
                ExamRecordForMarkingEntity examRecordForMarkingEntity = examRecordForMarkingRepo.findByExamRecordDataId(examRecordDataEntity.getId());
                offlineExamCourseInfo.setOfflineFileUrl(examRecordForMarkingEntity.getOfflineFileUrl());
                offlineExamCourseInfo.setFileName(examRecordForMarkingEntity.getOfflineFileName());
            }
        }
        return offlineExamCourseInfo;
    }

    @Override
    public void startOfflineExam(Long examStudentId) {
        SysPropertyCacheBean stuClientLoginLimit = CacheHelper.getSysProperty("STU_CLIENT_LOGIN_LIMIT");
        Boolean stuClientLoginLimitBoolean = false;
        if (stuClientLoginLimit.getHasValue()) {
            stuClientLoginLimitBoolean = Boolean.valueOf(stuClientLoginLimit.getValue().toString());
        }
        if (stuClientLoginLimitBoolean) {
            throw new StatusException("OE-001505", "系统维护中... ...");
        }
        List<ExamRecordDataEntity> examRecordList = examRecordDataRepo.findByExamStudentId(examStudentId);
        if (examRecordList != null && examRecordList.size() > 0) {
            throw new StatusException("OfflineExamServiceImpl-startOfflineExam-exception", "已经存在examStudentId=" + examStudentId + "的离线考试记录");
        }
        //获取考生信息
        ExamStudentEntity examStudentEntity = examStudentRepo.findByExamStudentId(examStudentId);
        //检查并获取课程信息
        CourseBean courseBean = checkCourse(examStudentEntity);
        //检查并获取考试信息
        ExamBean examBean = checkExam(examStudentEntity);
        //获取题库试卷结构（由于存在随机抽卷，所以不能缓存 ）
        GetPaperReq getPaperReq = new GetPaperReq();
        getPaperReq.setExamId(examStudentEntity.getExamId());
        getPaperReq.setCourseCode(courseBean.getCode());
        getPaperReq.setGroupCode(examStudentEntity.getPaperType());
        GetPaperResp getPaperResp = extractConfigCloudService.getPaper(getPaperReq);

        //生成考试记录
        ExamRecordDataEntity examRecordData = examRecordDataService.createOfflineExamRecordData(examStudentEntity,
                examBean, courseBean, getPaperResp.getPaperId(),
                null, getPaperResp.getDefaultPaper().getFullyObjective());
        //生成分数
        examScoreService.createExamScoreWithOffline(examRecordData.getId());
        //更新考生
        examStudentRepo.updateExamStudentFinished(examStudentId);
    }

    private CourseBean checkCourse(ExamStudentEntity examStudentEntity) {
        CourseBean courseBean = ExamCacheTransferHelper.getCachedCourse(examStudentEntity.getCourseId());
        if (!courseBean.getEnable()) {
            throw new StatusException("OfflineExamServiceImpl-checkCourse-exception", "该课程已被禁用");
        }
        return courseBean;
    }

    private ExamBean checkExam(ExamStudentEntity examStudentEntity) {
        ExamBean examBean = ExamCacheTransferHelper.getCachedExam(examStudentEntity.getExamId(),
                examStudentEntity.getOrgId());

        if (!examBean.getEnable() || (examBean.getExamLimit() != null && examBean.getExamLimit())) {
            throw new StatusException("ExamControlServiceImpl-checkExam-exception-01", "暂无考试资格,请与学校老师联系");
        }
        if (new Date().before(examBean.getBeginTime())) {
            throw new StatusException("ExamControlServiceImpl-checkExam-exception-02", "考试未开始");
        }
        if (examBean.getEndTime().before(new Date())) {
            throw new StatusException("ExamControlServiceImpl-checkExam-exception-03", "本次考试已结束");
        }
        return examBean;
    }

    @Override
    @Transactional
    public void submitPaper(Long examRecordDataId, File tempFile) throws Exception {
        SysPropertyCacheBean stuClientLoginLimit = CacheHelper.getSysProperty("STU_CLIENT_LOGIN_LIMIT");
        Boolean stuClientLoginLimitBoolean = false;
        if (stuClientLoginLimit.getHasValue()) {
            stuClientLoginLimitBoolean = Boolean.valueOf(stuClientLoginLimit.getValue().toString());
        }
        if (stuClientLoginLimitBoolean) {
            throw new StatusException("OE-001505", "系统维护中... ...");
        }
        ExamRecordDataEntity examRecordDataEntity = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);
        if (examRecordDataEntity == null) {
            return;
        }
        String fileName = tempFile.getName();
        String fileSuffix = fileName.substring(fileName.lastIndexOf(".") + 1, fileName.length()).toLowerCase();
        //上传文件至又拍云
        String fileNewName = createOfflineFileName(examRecordDataEntity) + "." + fileSuffix;

        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);
        String upyunFilePath = upyunUploadUrl + examRecordData.getExamId() + "/" + fileNewName;
        UpYun upyun = new UpYun(bucketName, userName, password);
        upyun.writeFile(upyunFilePath, tempFile, true);
        tempFile.delete();
        //保存 文件信息
        String fileUrl = upyunFileUrl + upyunFilePath;
        examRecordForMarkingService.saveOffLineExamRecordForMarking(examRecordDataEntity, fileNewName, fileUrl);
        //更新考试记录状态
        examRecordDataEntity.setExamRecordStatus(ExamRecordStatus.EXAM_END);
        examRecordDataEntity.setEndTime(new Date());//交卷(上传)时间
        examRecordDataRepo.save(examRecordDataEntity);
    }

    @Override
    public void downOfflineExam() {
        List<OfflineFileEntity> list = offlineFileRepo.findAll();
        for (int i = 0; i < list.size(); i++) {
            OfflineFileEntity fileEntity = list.get(i);
            String fileUrl = fileEntity.getFileUrl();
            System.out.println((i + 1) + "开始下载文件：" + fileEntity.getFileUrl());

            //对fileUrl中的中文进行编码
            fileUrl = encode(fileUrl, "utf-8");

            byte[] bs;
            try {
                bs = HttpClientUtil.get(fileUrl);
            }catch (Exception e){
                System.err.println((i + 1) + "文件：" + fileEntity.getFileUrl() + "下载失败,继续下一个文件...");
                e.printStackTrace();
                continue;
            }

            String suffix = fileUrl.substring(fileUrl.lastIndexOf("."));
            String filePath = "d:/data/" + fileEntity.getStudentCode() + "_" + fileEntity.getCourseCode() + suffix;
            File file = new File(filePath);
            try {
                FileUtils.writeByteArrayToFile(file, bs);
                System.out.println((i + 1) + "文件：" + fileEntity.getFileUrl() + "下载完毕...");
                System.out.println("剩余" + (list.size() - i - 1) + "个文件待下载...");
            } catch (IOException e) {
                throw new StatusException("100003", "文件读写失败");
            }
        }
    }

    private static String encode(String url, String chartSet) {
        try {
            Matcher matcher = Pattern.compile("[^\\x00-\\xff]").matcher(url);//双字节,包括中文和中文符号[^\x00-\xff]  中文[\u4e00-\u9fa5]
            while (matcher.find()) {
                String tmp = matcher.group();
                url = url.replaceAll(tmp, java.net.URLEncoder.encode(tmp, chartSet));
            }
        } catch (UnsupportedEncodingException e) {
            try {
                throw e;
            } catch (UnsupportedEncodingException e1) {
                e1.printStackTrace();
            }
        }
        return url;
    }

    private String createOfflineFileName(ExamRecordDataEntity examRecordDataEntity) {
        long currentTime = System.currentTimeMillis();

        long orgId = examRecordDataEntity.getOrgId();
        OrgCacheBean orgBean = gainBaseDataService.getOrgBean(orgId);
        long courseId = examRecordDataEntity.getCourseId();
        CourseBean courseBean = ExamCacheTransferHelper.getCachedCourse(courseId);
        return orgBean.getCode() + "_" +
                examRecordDataEntity.getStudentCode() + "_" +
                examRecordDataEntity.getStudentName() + "_" +
                courseBean.getCode() + "_" + currentTime;
    }


}
